# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import random
from datetime import datetime

from wechatpyyyy.pay.utils import get_external_ip
from wechatpyyyy.pay.base import BaseWeChatPayAPI


class WeChatRedpack(BaseWeChatPayAPI):

    def send(self, user_id, total_amount, send_name, act_name,
             wishing, remark, total_num=1, client_ip=None,
             out_trade_no=None, scene_id=None, consume_mch_id=None):
        """
        发送现金红包

        :param user_id: 接收红包的用户在公众号下的 openid
        :param total_amount: 红包金额，单位分
        :param send_name: 商户名称
        :param act_name: 活动名称
        :param wishing: 红包祝福语
        :param remark: 备注
        :param client_ip: 可选，调用接口的机器 IP 地址
        :param total_num: 可选，红包发放总人数，默认为 1
        :param out_trade_no: 可选，商户订单号，默认会自动生成
        :param scene_id: 可选，发放红包使用场景，红包金额大于200时必传
        :param consume_mch_id: 可选，资金授权商户号。服务商替特约商户发放时使用
        :return: 返回的结果数据字典
        """
        if not out_trade_no:
            now = datetime.now()
            out_trade_no = '{0}{1}{2}'.format(
                self.mch_id,
                now.strftime('%Y%m%d%H%M%S'),
                random.randint(1000, 10000)
            )
        data = {
            'wxappid': self.appid,
            're_openid': user_id,
            'total_amount': total_amount,
            'send_name': send_name,
            'act_name': act_name,
            'wishing': wishing,
            'remark': remark,
            'client_ip': client_ip or get_external_ip(),
            'total_num': total_num,
            'mch_billno': out_trade_no,
            'scene_id': scene_id,
            'risk_info': None,
            'consume_mch_id': consume_mch_id
        }
        return self._post('mmpaymkttransfers/sendredpack', data=data)

    def send_group(self, user_id, total_amount, send_name, act_name, wishing,
                   remark, total_num, client_ip=None, amt_type="ALL_RAND",
                   out_trade_no=None, scene_id=None, consume_mch_id=None):
        """
        发送裂变红包

        :param user_id: 接收红包的用户在公众号下的 openid
        :param total_amount: 红包金额，单位分
        :param send_name: 商户名称
        :param act_name: 活动名称
        :param wishing: 红包祝福语
        :param remark: 备注
        :param total_num: 红包发放总人数
        :param client_ip: 可选，调用接口的机器 IP 地址
        :param amt_type: 可选，红包金额设置方式
                         ALL_RAND—全部随机,商户指定总金额和红包发放总人数，由微信支付随机计算出各红包金额
        :param out_trade_no: 可选，商户订单号，默认会自动生成
        :param scene_id: 可选，发放红包使用场景，红包金额大于200时必传
        :param consume_mch_id: 可选，资金授权商户号。服务商替特约商户发放时使用
        :return: 返回的结果数据字典
        """
        if not out_trade_no:
            now = datetime.now()
            out_trade_no = '{0}{1}{2}'.format(
                self._client.mch_id,
                now.strftime('%Y%m%d%H%M%S'),
                random.randint(1000, 10000)
            )
        data = {
            'wxappid': self.appid,
            're_openid': user_id,
            'total_amount': total_amount,
            'send_name': send_name,
            'act_name': act_name,
            'wishing': wishing,
            'remark': remark,
            'total_num': total_num,
            'client_ip': client_ip or get_external_ip(),
            'amt_type': amt_type,
            'mch_billno': out_trade_no,
            'scene_id': scene_id,
            'risk_info': None,
            'consume_mch_id': consume_mch_id
        }
        return self._post('mmpaymkttransfers/sendgroupredpack', data=data)

    def query(self, out_trade_no, bill_type='MCHT'):
        """
        查询红包发放记录

        :param out_trade_no: 商户订单号
        :param bill_type: 可选，订单类型，目前固定为 MCHT
        :return: 返回的红包发放记录信息
        """
        data = {
            'mch_billno': out_trade_no,
            'bill_type': bill_type,
            'appid': self.appid,
        }
        return self._post('mmpaymkttransfers/gethbinfo', data=data)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import requests

from wechatpyyyy.client.api.base import BaseWeChatAPI


class WeChatMaterial(BaseWeChatAPI):

    def add_articles(self, articles):
        """
        新增永久图文素材
        详情请参考
        https://qydev.weixin.qq.com/wiki/index.php?title=%E4%B8%8A%E4%BC%A0%E6%B0%B8%E4%B9%85%E7%B4%A0%E6%9D%90

        :param articles: 图文素材数组
        :return: 返回的 JSON 数据包
        """
        articles_data = []
        for article in articles:
            articles_data.append({
                'thumb_media_id': article['thumb_media_id'],
                'title': article['title'],
                'content': article['content'],
                'author': article.get('author', ''),
                'content_source_url': article.get('content_source_url', ''),
                'digest': article.get('digest', ''),
                'show_cover_pic': article.get('show_cover_pic', 0)
            })
        return self._post(
            'material/add_mpnews',
            data={
                "mpnews": {
                    "articles": articles_data
                }
            }
        )

    def add(self, agent_id, media_type, media_file):
        """
        新增其它类型永久素材
        详情请参考
        https://qydev.weixin.qq.com/wiki/index.php?title=%E4%B8%8A%E4%BC%A0%E6%B0%B8%E4%B9%85%E7%B4%A0%E6%9D%90

        :param agent_id: 企业应用的id
        :param media_type: 媒体文件类型，分别有图片（image）、语音（voice）、视频（video）普通文件（file）
        :param media_file: 要上传的文件，一个 File-object
        :return: 返回的 JSON 数据包
        """
        params = {
            'agentid': agent_id,
            'type': media_type,
        }
        return self._post(
            url='material/add_material',
            params=params,
            files={
                'media': media_file
            }
        )

    def get_url(self, agent_id, media_id):
        """
        获取永久素材下载地址
        详情请参考
        https://qydev.weixin.qq.com/wiki/index.php?title=%E8%8E%B7%E5%8F%96%E6%B0%B8%E4%B9%85%E7%B4%A0%E6%9D%90

        :param agent_id: 企业应用的id
        :param media_id: 媒体文件 ID
        :return: 临时素材下载地址
        """
        parts = (
            'https://qyapi.weixin.qq.com/cgi-bin/material/get',
            '?access_token=',
            self.access_token,
            '&media_id=',
            media_id,
            '&agentid=',
            agent_id,
        )
        return ''.join(parts)

    def get(self, agent_id, media_id):
        """
        获取永久素材
        详情请参考
        https://qydev.weixin.qq.com/wiki/index.php?title=%E8%8E%B7%E5%8F%96%E6%B0%B8%E4%B9%85%E7%B4%A0%E6%9D%90

        :param agent_id: 企业应用的id
        :param media_id: 媒体文件 ID
        :return: requests 的 Response 实例
        """
        return requests.get(self.get_url(agent_id, media_id))

    def get_articles(self, agent_id, media_id):
        """
        获取永久素材：图文消息素材
        详情请参考
        https://qydev.weixin.qq.com/wiki/index.php?title=%E8%8E%B7%E5%8F%96%E6%B0%B8%E4%B9%85%E7%B4%A0%E6%9D%90

        :param agent_id: 企业应用的id
        :param media_id: 媒体文件 ID
        :return: 返回的 JSON 数据包
        """
        return self._get(
            'material/get',
            params={
                'agentid': agent_id,
                'media_id': media_id,
            }
        )

    def delete(self, agent_id, media_id):
        """
        删除永久素材
        详情请参考
        https://qydev.weixin.qq.com/wiki/index.php?title=%E5%88%A0%E9%99%A4%E6%B0%B8%E4%B9%85%E7%B4%A0%E6%9D%90

        :param agent_id: 企业应用的id
        :param media_id: 媒体文件 ID
        :return: 返回的 JSON 数据包
        """
        return self._get(
            'material/del',
            params={
                'agentid': agent_id,
                'media_id': media_id,
            }
        )

    def update_articles(self, agent_id, media_id, articles):
        """
        修改永久图文素材
        详情请参考
        https://qydev.weixin.qq.com/wiki/index.php?title=%E4%BF%AE%E6%94%B9%E6%B0%B8%E4%B9%85%E5%9B%BE%E6%96%87%E7%B4%A0%E6%9D%90

        :param media_id: 要修改的图文消息的 id
        :param index: 要更新的文章在图文消息中的位置（多图文消息时，此字段才有意义），第一篇为 0
        :param articles: 图文素材数组
        :return: 返回的 JSON 数据包
        """
        articles_data = []
        for article in articles:
            articles_data.append({
                'thumb_media_id': article['thumb_media_id'],
                'title': article['title'],
                'content': article['content'],
                'author': article.get('author', ''),
                'content_source_url': article.get('content_source_url', ''),
                'digest': article.get('digest', ''),
                'show_cover_pic': article.get('show_cover_pic', 0)
            })
        return self._post(
            'material/update_news',
            data={
                'agentid': agent_id,
                'media_id': media_id,
                'articles': articles_data
            }
        )

    def get_count(self, agent_id):
        """
        获取素材总数
        详情请参考
        https://qydev.weixin.qq.com/wiki/index.php?title=%E8%8E%B7%E5%8F%96%E7%B4%A0%E6%9D%90%E6%80%BB%E6%95%B0

        :param agent_id: 企业应用的id
        :return: 返回的 JSON 数据包
        """
        return self._get(
            'material/get_count',
            params={
                'agent_id': agent_id,
            }
        )

    def batchget(self, agent_id, media_type, offset=0, count=20):
        """
        批量获取永久素材列表
        详情请参考
        https://qydev.weixin.qq.com/wiki/index.php?title=%E8%8E%B7%E5%8F%96%E7%B4%A0%E6%9D%90%E5%88%97%E8%A1%A8

        :param agent_id: 企业应用的id
        :param media_type: 媒体文件类型，分别有图文（mpnews）、图片（image）、
                           语音（voice）、视频（video）和文件（file）
        :param offset: 从全部素材的该偏移位置开始返回，0 表示从第一个素材返回
        :param count: 返回素材的数量，取值在1到20之间
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'material/batchget',
            data={
                'agent_id': agent_id,
                'type': media_type,
                'offset': offset,
                'count': count
            }
        )



# -*- coding: utf-8 -*-
"""
    wechatpyyyy.parser
    ~~~~~~~~~~~~~~~~
    This module provides functions for parsing WeChat messages

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""
from __future__ import absolute_import, unicode_literals
import xmltodict

from wechatpyyyy.messages import MESSAGE_TYPES, UnknownMessage
from wechatpyyyy.events import EVENT_TYPES
from wechatpyyyy.utils import to_text


def parse_message(xml):
    """
    解析微信服务器推送的 XML 消息

    :param xml: XML 消息
    :return: 解析成功返回对应的消息或事件，否则返回 ``UnknownMessage``
    """
    if not xml:
        return
    message = xmltodict.parse(to_text(xml))['xml']
    message_type = message['MsgType'].lower()
    event_type = None
    if message_type == 'event' or message_type.startswith('device_'):
        if 'Event' in message:
            event_type = message['Event'].lower()
        # special event type for device_event
        if event_type is None and message_type.startswith('device_'):
            event_type = message_type
        elif message_type.startswith('device_'):
            event_type = 'device_{event}'.format(event=event_type)

        if event_type == 'subscribe' and message.get('EventKey'):
            event_key = message['EventKey']
            if event_key.startswith(('scanbarcode|', 'scanimage|')):
                event_type = 'subscribe_scan_product'
                message['Event'] = event_type
            elif event_key.startswith('qrscene_'):
                # Scan to subscribe with scene id event
                event_type = 'subscribe_scan'
                message['Event'] = event_type
                message['EventKey'] = event_key[len('qrscene_'):]
        message_class = EVENT_TYPES.get(event_type, UnknownMessage)
    else:
        message_class = MESSAGE_TYPES.get(message_type, UnknownMessage)
    return message_class(message)



# -*- coding: utf-8 -*-
"""
    wechatpyyyy.replies
    ~~~~~~~~~~~~~~~~~~
    This module defines all kinds of replies you can send to WeChat

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""
from __future__ import absolute_import, unicode_literals
import time
import six
import xmltodict

from wechatpyyyy.fields import (
    StringField,
    IntegerField,
    ImageField,
    VoiceField,
    VideoField,
    MusicField,
    ArticlesField,
    Base64EncodeField,
    HardwareField,
)
from wechatpyyyy.messages import BaseMessage, MessageMetaClass
from wechatpyyyy.utils import to_text, to_binary


REPLY_TYPES = {}


def register_reply(reply_type):
    def register(cls):
        REPLY_TYPES[reply_type] = cls
        return cls
    return register


class BaseReply(six.with_metaclass(MessageMetaClass)):
    """Base class for all replies"""
    source = StringField('FromUserName')
    target = StringField('ToUserName')
    time = IntegerField('CreateTime', time.time())
    type = 'unknown'

    def __init__(self, **kwargs):
        self._data = {}
        message = kwargs.pop('message', None)
        if message and isinstance(message, BaseMessage):
            if 'source' not in kwargs:
                kwargs['source'] = message.target
            if 'target' not in kwargs:
                kwargs['target'] = message.source
            if hasattr(message, 'agent') and 'agent' not in kwargs:
                kwargs['agent'] = message.agent
        if 'time' not in kwargs:
            kwargs['time'] = time.time()
        for name, value in kwargs.items():
            field = self._fields.get(name)
            if field:
                self._data[field.name] = value
            else:
                setattr(self, name, value)

    def render(self):
        """Render reply from Python object to XML string"""
        tpl = '<xml>\n{data}\n</xml>'
        nodes = []
        msg_type = '<MsgType><![CDATA[{msg_type}]]></MsgType>'.format(
            msg_type=self.type
        )
        nodes.append(msg_type)
        for name, field in self._fields.items():
            value = getattr(self, name, field.default)
            node_xml = field.to_xml(value)
            nodes.append(node_xml)
        data = '\n'.join(nodes)
        return tpl.format(data=data)

    def __str__(self):
        if six.PY2:
            return to_binary(self.render())
        else:
            return to_text(self.render())


@register_reply('empty')
class EmptyReply(BaseReply):
    """
    回复空串

    微信服务器不会对此作任何处理，并且不会发起重试
    """
    def __init__(self):
        pass

    def render(self):
        return ''


@register_reply('text')
class TextReply(BaseReply):
    """
    文本回复
    详情请参阅 http://mp.weixin.qq.com/wiki/9/2c15b20a16019ae613d413e30cac8ea1.html
    """
    type = 'text'
    content = StringField('Content')


@register_reply('image')
class ImageReply(BaseReply):
    """
    图片回复
    详情请参阅
    http://mp.weixin.qq.com/wiki/9/2c15b20a16019ae613d413e30cac8ea1.html
    """
    type = 'image'
    image = ImageField('Image')

    @property
    def media_id(self):
        return self.image

    @media_id.setter
    def media_id(self, value):
        self.image = value


@register_reply('voice')
class VoiceReply(BaseReply):
    """
    语音回复
    详情请参阅
    http://mp.weixin.qq.com/wiki/9/2c15b20a16019ae613d413e30cac8ea1.html
    """
    type = 'voice'
    voice = VoiceField('Voice')

    @property
    def media_id(self):
        return self.voice

    @media_id.setter
    def media_id(self, value):
        self.voice = value


@register_reply('video')
class VideoReply(BaseReply):
    """
    视频回复
    详情请参阅
    http://mp.weixin.qq.com/wiki/9/2c15b20a16019ae613d413e30cac8ea1.html
    """
    type = 'video'
    video = VideoField('Video', {})

    @property
    def media_id(self):
        return self.video.get('media_id')

    @media_id.setter
    def media_id(self, value):
        video = self.video
        video['media_id'] = value
        self.video = video

    @property
    def title(self):
        return self.video.get('title')

    @title.setter
    def title(self, value):
        video = self.video
        video['title'] = value
        self.video = video

    @property
    def description(self):
        return self.video.get('description')

    @description.setter
    def description(self, value):
        video = self.video
        video['description'] = value
        self.video = video


@register_reply('music')
class MusicReply(BaseReply):
    """
    音乐回复
    详情请参阅
    http://mp.weixin.qq.com/wiki/9/2c15b20a16019ae613d413e30cac8ea1.html
    """
    type = 'music'
    music = MusicField('Music', {})

    @property
    def thumb_media_id(self):
        return self.music.get('thumb_media_id')

    @thumb_media_id.setter
    def thumb_media_id(self, value):
        music = self.music
        music['thumb_media_id'] = value
        self.music = music

    @property
    def title(self):
        return self.music.get('title')

    @title.setter
    def title(self, value):
        music = self.music
        music['title'] = value
        self.music = music

    @property
    def description(self):
        return self.music.get('description')

    @description.setter
    def description(self, value):
        music = self.music
        music['description'] = value
        self.music = music

    @property
    def music_url(self):
        return self.music.get('music_url')

    @music_url.setter
    def music_url(self, value):
        music = self.music
        music['music_url'] = value
        self.music = music

    @property
    def hq_music_url(self):
        return self.music.get('hq_music_url')

    @hq_music_url.setter
    def hq_music_url(self, value):
        music = self.music
        music['hq_music_url'] = value
        self.music = music


@register_reply('news')
class ArticlesReply(BaseReply):
    """
    图文回复
    详情请参阅
    http://mp.weixin.qq.com/wiki/9/2c15b20a16019ae613d413e30cac8ea1.html
    """
    type = 'news'
    articles = ArticlesField('Articles', [])

    def add_article(self, article):
        if len(self.articles) == 10:
            raise AttributeError("Can't add more than 10 articles"
                                 " in an ArticlesReply")
        articles = self.articles
        articles.append(article)
        self.articles = articles


@register_reply('transfer_customer_service')
class TransferCustomerServiceReply(BaseReply):
    """
    将消息转发到多客服
    详情请参阅
    http://mp.weixin.qq.com/wiki/5/ae230189c9bd07a6b221f48619aeef35.html
    """
    type = 'transfer_customer_service'


@register_reply('device_text')
class DeviceTextReply(BaseReply):
    type = 'device_text'
    device_type = StringField('DeviceType')
    device_id = StringField('DeviceID')
    session_id = StringField('SessionID')
    content = Base64EncodeField('Content')


@register_reply('device_event')
class DeviceEventReply(BaseReply):
    type = 'device_event'
    event = StringField('Event')
    device_type = StringField('DeviceType')
    device_id = StringField('DeviceID')
    session_id = StringField('SessionID')
    content = Base64EncodeField('Content')


@register_reply('device_status')
class DeviceStatusReply(BaseReply):
    type = 'device_status'
    device_type = StringField('DeviceType')
    device_id = StringField('DeviceID')
    status = IntegerField('DeviceStatus')


@register_reply('hardware')
class HardwareReply(BaseReply):
    type = 'hardware'
    func_flag = IntegerField('FuncFlag', 0)
    hardware = HardwareField('HardWare')


def create_reply(reply, message=None, render=False):
    """
    Create a reply quickly
    """
    r = None
    if not reply:
        r = EmptyReply()
    elif isinstance(reply, BaseReply):
        r = reply
        if message:
            r.source = message.target
            r.target = message.source
    elif isinstance(reply, six.string_types):
        r = TextReply(
            message=message,
            content=reply
        )
    elif isinstance(reply, (tuple, list)):
        if len(reply) > 10:
            raise AttributeError("Can't add more than 10 articles"
                                 " in an ArticlesReply")
        r = ArticlesReply(
            message=message,
            articles=reply
        )
    if r and render:
        return r.render()
    return r


def deserialize_reply(xml, update_time=False):
    """
    反序列化被动回复
    :param xml: 待反序列化的xml
    :param update_time: 是否用当前时间替换xml中的时间
    :raises ValueError: 不能辨识的reply xml
    :rtype: wechatpyyyy.replies.BaseReply
    """
    if not xml:
        return EmptyReply()

    try:
        reply_dict = xmltodict.parse(xml)["xml"]
        msg_type = reply_dict["MsgType"]
    except (xmltodict.expat.ExpatError, KeyError):
        raise ValueError("bad reply xml")
    if msg_type not in REPLY_TYPES:
        raise ValueError("unknown reply type")

    cls = REPLY_TYPES[msg_type]
    kwargs = dict()
    for attr, field in cls._fields.items():
        if field.name in reply_dict:
            str_value = reply_dict[field.name]
            kwargs[attr] = field.from_xml(str_value)

    if update_time:
        kwargs["time"] = time.time()

    return cls(**kwargs)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from wechatpyyyy.client.api.base import BaseWeChatAPI


class MerchantGroup(BaseWeChatAPI):

    API_BASE_URL = 'https://api.weixin.qq.com/'

    def add(self, name, product_list):
        return self._post(
            'merchant/group/add',
            data={
                'group_detail': {
                    'group_name': name,
                    'product_list': product_list
                }
            }
        )

    def delete(self, group_id):
        return self._post(
            'merchant/group/del',
            data={
                'group_id': group_id
            }
        )

    def update(self, group_id, name):
        return self._post(
            'merchant/group/propertymod',
            data={
                'group_id': group_id,
                'group_name': name
            }
        )

    def update_product(self, group_id, product):
        return self._post(
            'merchant/group/productmod',
            data={
                'group_id': group_id,
                'product': product
            }
        )

    def get_all(self):
        res = self._get(
            'merchant/group/getall',
            result_processor=lambda x: x['groups_detail']
        )
        return res

    def get(self, group_id):
        res = self._post(
            'merchant/group/getbyid',
            data={
                'group_id': group_id
            },
            result_processor=lambda x: x['group_detail']
        )
        return res



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import base64
import urllib

from wechatpyyyy.client.api.base import BaseWeChatAPI
from wechatpyyyy.utils import to_binary, to_text


class WeChatDevice(BaseWeChatAPI):
    API_BASE_URL = 'https://api.weixin.qq.com/device/'

    def send_message(self, device_type, device_id, user_id, content):
        """
        主动发送消息给设备
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-3

        :param device_type: 设备类型，目前为“公众账号原始ID”
        :param device_id: 设备ID
        :param user_id: 微信用户账号的openid
        :param content: 消息内容，BASE64编码
        :return: 返回的 JSON 数据包
        """
        content = to_text(base64.b64encode(to_binary(content)))
        return self._post(
            'transmsg',
            data={
                'device_type': device_type,
                'device_id': device_id,
                'open_id': user_id,
                'content': content
            }
        )

    def send_status_message(self, device_type, device_id, user_id, msg_type, device_status):
        """
        第三方主动发送设备状态消息给微信终端
        详情请参考
        https://iot.weixin.qq.com/wiki/document-2_10.html

        :param device_type: 设备类型，目前为“公众账号原始ID”
        :param device_id: 设备ID
        :param user_id: 微信用户账号的openid
        :param msg_type: 消息类型：2--设备状态消息
        :param status: 设备状态：0--未连接， 1--已连接
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'transmsg',
            data={
                'device_type': device_type,
                'device_id': device_id,
                'open_id': user_id,
                'msg_type': msg_type,
                'device_status': device_status,
            }
        )

    def create_qrcode(self, device_ids):
        """
        获取设备二维码
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-4

        :param device_ids: 设备id的列表
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'create_qrcode',
            data={
                'device_num': len(device_ids),
                'device_id_list': device_ids
            }
        )

    def get_qrcode_url(self, ticket, data=None):
        """
        通过 ticket 换取二维码地址
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-4

        :param ticket: 二维码 ticket
        :param data: 额外数据
        :return: 二维码地址
        """
        url = 'https://we.qq.com/d/{ticket}'.format(ticket=ticket)
        if data:
            if isinstance(data, (dict, tuple, list)):
                data = urllib.urlencode(data)
            data = to_text(base64.b64encode(to_binary(data)))
            url = '{base}#{data}'.format(base=url, data=data)
        return url

    def bind(self, ticket, device_id, user_id):
        """
        绑定设备
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-7

        :param ticket: 绑定操作合法性的凭证（由微信后台生成，第三方H5通过客户端jsapi获得）
        :param device_id: 设备id
        :param user_id: 用户对应的openid
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'bind',
            data={
                'ticket': ticket,
                'device_id': device_id,
                'openid': user_id
            }
        )

    def unbind(self, ticket, device_id, user_id):
        """
        解绑设备
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-7

        :param ticket: 绑定操作合法性的凭证（由微信后台生成，第三方H5通过客户端jsapi获得）
        :param device_id: 设备id
        :param user_id: 用户对应的openid
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'unbind',
            data={
                'ticket': ticket,
                'device_id': device_id,
                'openid': user_id
            }
        )

    def compel_bind(self, device_id, user_id):
        """
        强制绑定用户和设备
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-7

        :param device_id: 设备id
        :param user_id: 用户对应的openid
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'compel_bind',
            data={
                'device_id': device_id,
                'openid': user_id
            }
        )

    force_bind = compel_bind

    def compel_unbind(self, device_id, user_id):
        """
        强制解绑用户和设备
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-7

        :param device_id: 设备id
        :param user_id: 用户对应的openid
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'compel_unbind',
            data={
                'device_id': device_id,
                'openid': user_id
            }
        )

    force_unbind = compel_unbind

    def get_stat(self, device_id):
        """
        设备状态查询
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-8

        :param device_id: 设备id
        :return: 返回的 JSON 数据包
        """
        return self._get(
            'get_stat',
            params={'device_id': device_id}
        )

    def verify_qrcode(self, ticket):
        """
        验证二维码
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-9

        :param ticket: 设备二维码的ticket
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'verify_qrcode',
            data={'ticket': ticket}
        )

    def get_user_id(self, device_type, device_id):
        """
        获取设备绑定openID
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-11

        :param device_type: 设备类型，目前为“公众账号原始ID”
        :param device_id: 设备id
        :return: 返回的 JSON 数据包
        """
        return self._get(
            'get_openid',
            params={
                'device_type': device_type,
                'device_id': device_id
            }
        )

    get_open_id = get_user_id

    def get_binded_devices(self, user_id):
        """
        通过openid获取用户在当前devicetype下绑定的deviceid列表
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-12

        :param user_id: 要查询的用户的openid
        :return: 返回的 JSON 数据包
        """
        return self._get(
            'get_bind_device',
            params={'openid': user_id}
        )

    get_bind_device = get_binded_devices

    def get_qrcode(self, product_id=1):
        """
        获取deviceid和二维码
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-4

        :param product_id: 设备的产品编号
        :return: 返回的 JSON 数据包
        """
        if product_id == '1' or product_id == 1:
            params = None
        else:
            params = {'product_id': product_id}

        return self._get('getqrcode', params=params)

    def authorize(self, devices, op_type=1):
        """
        设备授权
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-5

        :param devices: 设备信息的列表
        :param op_type: 请求操作的类型，限定取值为：0：设备授权 1：设备更新
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'authorize_device',
            data={
                'device_num': len(devices),
                'device_list': devices,
                'op_type': op_type
            }
        )



# -*- coding: utf-8 -*-
"""
    wechatpyyyy.client.jsapi
    ~~~~~~~~~~~~~~~~~~~~

    This module provides some APIs for JS SDK

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""
from __future__ import absolute_import, unicode_literals

import hashlib
import time

from wechatpyyyy.utils import WeChatSigner, random_string
from wechatpyyyy.client.api.base import BaseWeChatAPI


class WeChatJSAPI(BaseWeChatAPI):

    def get_ticket(self, type='jsapi'):
        """
        获取微信 JS-SDK ticket

        :return: 返回的 JSON 数据包
        """
        return self._get(
            'ticket/getticket',
            params={'type': type}
        )

    def get_jsapi_ticket(self):
        """
        获取微信 JS-SDK ticket

        该方法会通过 session 对象自动缓存管理 ticket

        :return: ticket
        """
        ticket_key = '{0}_jsapi_ticket'.format(self.appid)
        expires_at_key = '{0}_jsapi_ticket_expires_at'.format(self.appid)
        ticket = self.session.get(ticket_key)
        expires_at = self.session.get(expires_at_key, 0)
        if not ticket or expires_at < int(time.time()):
            jsapi_ticket_response = self.get_ticket('jsapi')
            ticket = jsapi_ticket_response['ticket']
            expires_at = int(time.time()) + int(jsapi_ticket_response['expires_in'])
            self.session.set(ticket_key, ticket)
            self.session.set(expires_at_key, expires_at)
        return ticket

    def get_jsapi_signature(self, noncestr, ticket, timestamp, url):
        """
        获取 JSAPI 签名

        :param noncestr: nonce string
        :param ticket: JS-SDK ticket
        :param timestamp: 时间戳
        :param url: URL
        :return: 签名
        """
        data = [
            'noncestr={noncestr}'.format(noncestr=noncestr),
            'jsapi_ticket={ticket}'.format(ticket=ticket),
            'timestamp={timestamp}'.format(timestamp=timestamp),
            'url={url}'.format(url=url),
        ]
        signer = WeChatSigner(delimiter=b'&')
        signer.add_data(*data)
        return signer.signature

    def get_jsapi_card_ticket(self):
        """
        获取 api_ticket：是用于调用微信卡券JS API的临时票据，有效期为7200 秒，通过access_token 来获取。
        微信文档地址：https://developers.weixin.qq.com/doc/offiaccount/OA_Web_Apps/JS-SDK.html#62
        该方法会通过 session 对象自动缓存管理 ticket

        :return: ticket
        """
        jsapi_card_ticket_key = '{0}_jsapi_card_ticket'.format(self.appid)
        jsapi_card_ticket_expire_at_key = '{0}_jsapi_card_ticket_expires_at'.format(self.appid)

        ticket = self.session.get(jsapi_card_ticket_key)
        expires_at = self.session.get(jsapi_card_ticket_expire_at_key, 0)
        if not ticket or int(expires_at) < int(time.time()):
            ticket_response = self.get_ticket('wx_card')
            ticket = ticket_response['ticket']
            expires_at = int(time.time()) + int(ticket_response['expires_in'])
            self.session.set(jsapi_card_ticket_key, ticket)
            self.session.set(jsapi_card_ticket_expire_at_key, expires_at)
        return ticket

    def get_jsapi_card_params(self, card_ticket, card_type, **kwargs):
        """
        参数意义见微信文档地址：https://developers.weixin.qq.com/doc/offiaccount/OA_Web_Apps/JS-SDK.html#62
        :param card_ticket: 用于卡券的微信 api_ticket
        :param card_type:
        :param kwargs: 非必须参数：noncestr, timestamp, code, openid, fixed_begintimestamp, outer_str
        :return: 包含调用jssdk所有所需参数的 dict
        """
        card_signature_dict = {
            'card_type': card_type,
            'noncestr': kwargs.get('noncestr', random_string()),
            'api_ticket': card_ticket,
            'appid': self.appid,
            'timestamp': kwargs.get('timestamp', str(int(time.time()))),
        }
        list_before_sign = sorted([str(x) for x in card_signature_dict.values()])
        str_to_sign = "".join(list_before_sign).encode()
        card_signature_dict['sign'] = hashlib.sha1(str_to_sign).hexdigest()
        return card_signature_dict



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from wechatpyyyy.client.api.base import BaseWeChatAPI


class MerchantExpress(BaseWeChatAPI):

    API_BASE_URL = 'https://api.weixin.qq.com/'

    def add(self, delivery_template):
        return self._post(
            'merchant/express/add',
            data={
                'delivery_template': delivery_template
            }
        )

    def delete(self, template_id):
        return self._post(
            'merchant/express/del',
            data={
                'template_id': template_id
            }
        )

    def update(self, template_id, delivery_template):
        return self._post(
            'merchant/express/update',
            data={
                'template_id': template_id,
                'delivery_template': delivery_template
            }
        )

    def get(self, template_id):
        res = self._post(
            'merchant/express/getbyid',
            data={
                'template_id': template_id
            },
            result_processor=lambda x: x['template_info']
        )
        return res

    def get_all(self):
        res = self._get(
            'merchant/express/getall',
            result_processor=lambda x: x['template_info']
        )
        return res



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from optionaldict import optionaldict

from wechatpyyyy.client.api.base import BaseWeChatAPI


class WeChatAgent(BaseWeChatAPI):
    """
    https://work.weixin.qq.com/api/doc#90000/90135/90226
    """

    def get(self, agent_id):
        """
        获取指定的应用详情
        https://work.weixin.qq.com/api/doc#90000/90135/90227/获取指定的应用详情/

        :param agent_id: 应用id
        :return: 返回的 JSON 数据包
        """
        return self._get(
            'agent/get',
            params={
                'agentid': agent_id
            }
        )

    def list(self):
        """
        获取access_token对应的应用列表
        https://work.weixin.qq.com/api/doc#90000/90135/90227/获取access_token对应的应用列表/

        :return: 应用概况列表
        """
        res = self._get('agent/list')
        return res['agentlist']

    def set(self,
            agent_id,
            name=None,
            description=None,
            redirect_domain=None,
            logo_media_id=None,
            report_location_flag=0,
            is_report_user=True,
            is_report_enter=True):
        """
        设置应用
        https://work.weixin.qq.com/api/doc#90000/90135/90228

        :param agent_id: 企业应用的id
        :param name: 企业应用名称，长度不超过32个utf8字符
        :param description: 企业应用详情，长度为4至120个utf8字符
        :param redirect_domain: 企业应用可信域名。注意：域名需通过所有权校验，否则jssdk功能将受限，此时返回错误码85005
        :param logo_media_id: 企业应用头像的mediaid，通过素材管理接口上传图片获得mediaid，上传后会自动裁剪成方形和圆形两个头像
        :param report_location_flag: 企业应用是否打开地理位置上报 0：不上报；1：进入会话上报；
        :param is_report_enter: 是否上报用户进入应用事件。0：不接收；1：接收。
        :param is_report_user: 是否接收用户变更通知。0：不接收；1：接收。
        :return: 返回的 JSON 数据包
        """
        agent_data = optionaldict()
        agent_data['agentid'] = agent_id
        agent_data['name'] = name
        agent_data['description'] = description
        agent_data['redirect_domain'] = redirect_domain
        agent_data['logo_mediaid'] = logo_media_id
        agent_data['report_location_flag'] = report_location_flag
        agent_data['isreportenter'] = 1 if is_report_enter else 0
        agent_data['isreportuser'] = 1 if is_report_user else 0
        return self._post(
            'agent/set',
            data=agent_data
        )



# -*- coding: utf-8 -*-
"""
    wechatpyyyy.events
    ~~~~~~~~~~~~~~~~

    This module contains all the events WeChat callback uses.

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""
from __future__ import absolute_import, unicode_literals

from wechatpyyyy.fields import (
    Base64DecodeField,
    BaseField,
    DateTimeField,
    FloatField,
    IntegerField,
    StringField
)
from wechatpyyyy.messages import BaseMessage

EVENT_TYPES = {}


def register_event(event_type):
    """
    Register the event class so that they can be accessed from EVENT_TYPES

    :param event_type: Event type
    """

    def register(cls):
        EVENT_TYPES[event_type] = cls
        return cls

    return register


class BaseEvent(BaseMessage):
    """Base class for all events"""
    type = 'event'
    event = ''


@register_event('subscribe')
class SubscribeEvent(BaseEvent):
    """
    用户关注事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1421140454
    """
    event = 'subscribe'
    key = StringField('EventKey', '')


@register_event('unsubscribe')
class UnsubscribeEvent(BaseEvent):
    """
    用户取消关注事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1421140454
    """
    event = 'unsubscribe'


@register_event('subscribe_scan')
class SubscribeScanEvent(BaseEvent):
    """
    用户扫描二维码关注事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1421140454
    """
    event = 'subscribe_scan'
    scene_id = StringField('EventKey')
    ticket = StringField('Ticket')


@register_event('scan')
class ScanEvent(BaseEvent):
    """
    用户扫描二维码事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1421140454
    """
    event = 'scan'
    scene_id = StringField('EventKey')
    ticket = StringField('Ticket')


@register_event('location')
class LocationEvent(BaseEvent):
    """
    上报地理位置事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1421140454
    """
    event = 'location'
    latitude = FloatField('Latitude', 0.0)
    longitude = FloatField('Longitude', 0.0)
    precision = FloatField('Precision', 0.0)


@register_event('click')
class ClickEvent(BaseEvent):
    """
    点击菜单拉取消息事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1421140454
    """
    event = 'click'
    key = StringField('EventKey')


@register_event('view')
class ViewEvent(BaseEvent):
    """
    点击菜单跳转链接事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1421140454
    """
    event = 'view'
    url = StringField('EventKey')


@register_event('masssendjobfinish')
class MassSendJobFinishEvent(BaseEvent):
    """
    群发消息任务完成事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1481187827_i0l21
    """
    id = IntegerField('MsgID', 0)
    event = 'masssendjobfinish'
    status = StringField('Status')
    total_count = IntegerField('TotalCount', 0)
    filter_count = IntegerField('FilterCount', 0)
    sent_count = IntegerField('SentCount', 0)
    error_count = IntegerField('ErrorCount', 0)


@register_event('templatesendjobfinish')
class TemplateSendJobFinishEvent(BaseEvent):
    """
    模板消息任务完成事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1433751277
    """
    id = IntegerField('MsgID')
    event = 'templatesendjobfinish'
    status = StringField('Status')


class BaseScanCodeEvent(BaseEvent):
    key = StringField('EventKey')
    scan_code_info = BaseField('ScanCodeInfo', {})

    @property
    def scan_type(self):
        return self.scan_code_info['ScanType']

    @property
    def scan_result(self):
        return self.scan_code_info['ScanResult']


@register_event('scancode_push')
class ScanCodePushEvent(BaseScanCodeEvent):
    """
    扫码推事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1421141016
    """
    event = 'scancode_push'


@register_event('scancode_waitmsg')
class ScanCodeWaitMsgEvent(BaseScanCodeEvent):
    """
    扫码推事件且弹出“消息接收中”提示框的事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1421141016
    """
    event = 'scancode_waitmsg'


class BasePictureEvent(BaseEvent):
    key = StringField('EventKey')
    pictures_info = BaseField('SendPicsInfo', {})

    @property
    def count(self):
        return int(self.pictures_info['Count'])

    @property
    def pictures(self):
        if self.pictures_info['PicList']:
            items = self.pictures_info['PicList']['item']
            if self.count > 1:
                return items
            return [items]
        return []


@register_event('pic_sysphoto')
class PicSysPhotoEvent(BasePictureEvent):
    """
    弹出系统拍照发图的事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1421141016
    """
    event = 'pic_sysphoto'


@register_event('pic_photo_or_album')
class PicPhotoOrAlbumEvent(BasePictureEvent):
    """
    弹出拍照或者相册发图的事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1421141016
    """
    event = 'pic_photo_or_album'


@register_event('pic_weixin')
class PicWeChatEvent(BasePictureEvent):
    """
    弹出微信相册发图器的事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1421141016
    """
    event = 'pic_weixin'


@register_event('location_select')
class LocationSelectEvent(BaseEvent):
    """
    弹出地理位置选择器的事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1421141016
    """
    event = 'location_select'
    key = StringField('EventKey')
    location_info = BaseField('SendLocationInfo', {})

    @property
    def location_x(self):
        return self.location_info['Location_X']

    @property
    def location_y(self):
        return self.location_info['Location_Y']

    @property
    def location(self):
        return self.location_x, self.location_y

    @property
    def scale(self):
        return self.location_info['Scale']

    @property
    def label(self):
        return self.location_info['Label']

    @property
    def poiname(self):
        return self.location_info['Poiname']


@register_event('card_pass_check')
class CardPassCheckEvent(BaseEvent):
    """
    卡券审核事件推送

    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#1
    """
    event = 'card_pass_check'
    card_id = StringField('CardId')


@register_event('card_not_pass_check')
class CardNotPassCheckEvent(BaseEvent):
    event = 'card_not_pass_check'
    card_id = StringField('CardId')
    refuse_reason = StringField('RefuseReason')


@register_event('user_get_card')
class UserGetCardEvent(BaseEvent):
    """
    领取事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#2
    """
    event = 'user_get_card'
    card_id = StringField('CardId')
    is_given_by_friend = IntegerField('IsGiveByFriend')
    friend = StringField('FriendUserName')
    code = StringField('UserCardCode')
    old_code = StringField('OldUserCardCode')
    outer_id = StringField('OuterId')
    outer_str = StringField('OuterStr')
    is_restore_member_card = IntegerField('IsRestoreMemberCard')
    is_recommend_by_friend = IntegerField('IsRecommendByFriend')
    union_id = StringField('UnionId')


@register_event('user_gifting_card')
class UserGiftingCardEvent(BaseEvent):
    """
    转赠事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#3
    """
    event = 'user_gifting_card'
    card_id = StringField('CardId')
    code = StringField('UserCardCode')
    is_return_back = IntegerField('IsReturnBack')
    friend = StringField('FriendUserName')
    is_chat_room = IntegerField('IsChatRoom')


@register_event('user_del_card')
class UserDeleteCardEvent(BaseEvent):
    """
    卡券删除事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#4
    """
    event = 'user_del_card'
    card_id = StringField('CardId')
    code = StringField('UserCardCode')


@register_event('user_consume_card')
class UserConsumeCardEvent(BaseEvent):
    """
    卡券核销事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#5
    """
    event = 'user_consume_card'
    card_id = StringField('CardId')
    code = StringField('UserCardCode')
    consume_source = StringField('ConsumeSource')
    location_name = StringField('LocationName')
    staff = StringField('StaffOpenId')
    verify_code = StringField('VerifyCode')
    remark_amount = StringField('RemarkAmount')
    outer_str = StringField('OuterStr')


@register_event('user_pay_from_pay_cell')
class UserPayFromPayCell(BaseEvent):
    """
    卡券买单事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#6
    """
    event = 'user_pay_from_pay_cell'
    card_id = StringField('CardId')
    code = StringField('UserCardCode')
    trans_id = StringField('TransId')
    location_id = IntegerField('LocationId')
    fee = IntegerField('Fee')
    original_fee = IntegerField('OriginalFee')


@register_event('user_view_card')
class UserViewCard(BaseEvent):
    """
    进入会员卡事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#7
    """
    event = 'user_view_card'
    card_id = StringField('CardId')
    code = StringField('UserCardCode')
    outer_str = StringField('OuterStr')


@register_event('user_enter_session_from_card')
class UserEnterSessionFromCardEvent(BaseEvent):
    """
    从卡券进入公众号会话事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#8
    """
    event = 'user_enter_session_from_card'
    card_id = StringField('CardId')
    code = StringField('UserCardCode')


@register_event('update_member_card')
class UpdateMemberCard(BaseEvent):
    """
    会员卡内容更新事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#9
    """
    event = 'update_member_card'
    card_id = StringField('CardId')
    code = StringField('UserCardCode')
    modify_bonus = IntegerField('ModifyBonus')
    modify_balance = IntegerField('ModifyBalance')


@register_event('card_sku_remind')
class CardSkuRemindEvent(BaseEvent):
    """
    卡券库存报警事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#10
    """
    event = 'card_sku_remind'
    card_id = StringField('CardId')
    detail = StringField('Detail')


@register_event('card_pay_order')
class CardPayOrderEvent(BaseEvent):
    """
    券点流水详情事件

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#11
    """
    event = 'card_pay_order'
    order_id = IntegerField('OrderId')
    status = StringField('Status')
    create_order_time = IntegerField('CreateOrderTime')
    pay_finish_time = IntegerField('PayFinishTime')
    description = StringField('Desc')
    free_coin_count = IntegerField('FreeCoinCount')
    pay_coin_count = IntegerField('PayCoinCount')
    refund_free_coin_count = IntegerField('RefundFreeCoinCount')
    refund_pay_coin_count = IntegerField('RefundPayCoinCount')
    order_type = StringField('OrderType')
    memo = StringField('Memo')
    receipt_info = StringField('ReceiptInfo')


@register_event('submit_membercard_user_info')
class SubmitMembercardUserInfo(BaseEvent):
    """
    会员卡激活事件推送

    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Coupons_Vouchers_and_Cards_Event_Push_Messages.html#12
    """
    event = 'submit_membercard_user_info'
    card_id = StringField('CardId')
    card_code = StringField('UserCardCode')


@register_event('merchant_order')
class MerchantOrderEvent(BaseEvent):
    event = 'merchant_order'
    order_id = StringField('OrderId')
    order_status = IntegerField('OrderStatus')
    product_id = StringField('ProductId')
    sku_info = StringField('SkuInfo')


@register_event('kf_create_session')
class KfCreateSessionEvent(BaseEvent):
    event = 'kf_create_session'
    account = StringField('KfAccount')


@register_event('kf_close_session')
class KfCloseSessionEvent(BaseEvent):
    event = 'kf_close_session'
    account = StringField('KfAccount')


@register_event('kf_switch_session')
class KfSwitchSessionEvent(BaseEvent):
    event = 'kf_switch_session'
    from_account = StringField('FromKfAccount')
    to_account = StringField('ToKfAccount')


@register_event('device_text')
class DeviceTextEvent(BaseEvent):
    event = 'device_text'
    device_type = StringField('DeviceType')
    device_id = StringField('DeviceID')
    session_id = StringField('SessionID')
    content = Base64DecodeField('Content')
    open_id = StringField('OpenID')


@register_event('device_bind')
class DeviceBindEvent(BaseEvent):
    event = 'device_bind'
    device_type = StringField('DeviceType')
    device_id = StringField('DeviceID')
    session_id = StringField('SessionID')
    content = Base64DecodeField('Content')
    open_id = StringField('OpenID')


@register_event('device_unbind')
class DeviceUnbindEvent(BaseEvent):
    event = 'device_unbind'
    device_type = StringField('DeviceType')
    device_id = StringField('DeviceID')
    session_id = StringField('SessionID')
    content = Base64DecodeField('Content')
    open_id = StringField('OpenID')


@register_event('device_subscribe_status')
class DeviceSubscribeStatusEvent(BaseEvent):
    event = 'device_subscribe_status'
    device_type = StringField('DeviceType')
    device_id = StringField('DeviceID')
    open_id = StringField('OpenID')
    op_type = IntegerField('OpType')


@register_event('device_unsubscribe_status')
class DeviceUnsubscribeStatusEvent(BaseEvent):
    event = 'device_unsubscribe_status'
    device_type = StringField('DeviceType')
    device_id = StringField('DeviceID')
    open_id = StringField('OpenID')
    op_type = IntegerField('OpType')


@register_event('shakearoundusershake')
class ShakearoundUserShakeEvent(BaseEvent):
    event = 'shakearound_user_shake'
    _chosen_beacon = BaseField('ChosenBeacon', {})
    _around_beacons = BaseField('AroundBeacons', {})

    @property
    def chosen_beacon(self):
        beacon = self._chosen_beacon
        if not beacon:
            return {}
        return {
            'uuid': beacon['Uuid'],
            'major': beacon['Major'],
            'minor': beacon['Minor'],
            'distance': float(beacon['Distance']),
        }

    @property
    def around_beacons(self):
        beacons = self._around_beacons
        if not beacons:
            return []

        ret = []
        for beacon in beacons['AroundBeacon']:
            ret.append({
                'uuid': beacon['Uuid'],
                'major': beacon['Major'],
                'minor': beacon['Minor'],
                'distance': float(beacon['Distance']),
            })
        return ret


@register_event('poi_check_notify')
class PoiCheckNotifyEvent(BaseEvent):
    event = 'poi_check_notify'
    poi_id = StringField('PoiId')
    uniq_id = StringField('UniqId')
    result = StringField('Result')
    message = StringField('Msg')


@register_event('wificonnected')
class WiFiConnectedEvent(BaseEvent):
    event = 'wificconnected'
    connect_time = IntegerField('ConnectTime')
    expire_time = IntegerField('ExpireTime')
    vendor_id = StringField('VendorId')
    shop_id = StringField('PlaceId')
    bssid = StringField('DeviceNo')


# ============================================================================
# 微信认证事件推送
# ============================================================================
@register_event('qualification_verify_success')
class QualificationVerifySuccessEvent(BaseEvent):
    """
    资质认证成功事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1455785130
    """
    event = 'qualification_verify_success'
    expired_time = DateTimeField('ExpiredTime')


@register_event('qualification_verify_fail')
class QualificationVerifyFailEvent(BaseEvent):
    """
    资质认证失败事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1455785130
    """
    event = 'qualification_verify_fail'
    fail_time = DateTimeField('FailTime')
    fail_reason = StringField('FailReason')


@register_event('naming_verify_success')
class NamingVerifySuccessEvent(BaseEvent):
    """
    名称认证成功事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1455785130
    """
    event = 'naming_verify_success'
    expired_time = DateTimeField('ExpiredTime')


@register_event('naming_verify_fail')
class NamingVerifyFailEvent(BaseEvent):
    """
    名称认证失败事件

    客户端不打勾，但仍有接口权限。详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1455785130
    """
    event = 'naming_verify_fail'
    fail_time = DateTimeField('FailTime')
    fail_reason = StringField('FailReason')


@register_event('annual_renew')
class AnnualRenewEvent(BaseEvent):
    """
    年审通知事件

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1455785130
    """
    event = 'annual_renew'
    expired_time = DateTimeField('ExpiredTime')


@register_event('verify_expired')
class VerifyExpiredEvent(BaseEvent):
    """
    认证过期失效通知

    详情请参阅
    https://mp.weixin.qq.com/wiki?id=mp1455785130
    """
    event = 'verify_expired'
    expired_time = DateTimeField('ExpiredTime')


@register_event('user_scan_product')
class UserScanProductEvent(BaseEvent):
    """
    打开商品主页事件

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1455872179
    """
    event = 'user_scan_product'
    standard = StringField('KeyStandard')
    key = StringField('KeyStr')
    country = StringField('Country')
    province = StringField('Province')
    city = StringField('City')
    sex = IntegerField('Sex')
    scene = IntegerField('Scene')


@register_event('user_scan_product_enter_session')
class UserScanProductEnterSessionEvent(BaseEvent):
    """
    进入公众号事件

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1455872179
    """
    event = 'user_scan_product_enter_session'
    standard = StringField('KeyStandard')
    key = StringField('KeyStr')


@register_event('user_scan_product_async')
class UserScanProductAsyncEvent(BaseEvent):
    """
    地理位置信息异步推送事件

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1455872179
    """
    event = 'user_scan_product_async'
    standard = StringField('KeyStandard')
    key = StringField('KeyStr')
    region_code = StringField('RegionCode')


@register_event('user_scan_product_verify_action')
class UserScanProductVerifyActionEvent(BaseEvent):
    """
    商品审核结果事件

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1455872179
    """
    event = 'user_scan_product_verify_action'
    standard = StringField('KeyStandard')
    key = StringField('KeyStr')
    result = StringField('Result')
    reason = StringField('ReasonMsg')


@register_event('subscribe_scan_product')
class SubscribeScanProductEvent(BaseEvent):
    """
    用户在商品主页中关注公众号事件

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1455872179
    """
    event = 'subscribe_scan_product'
    event_key = StringField('EventKey')

    @property
    def scene(self):
        return self.event_key.split('|', 1)[0]

    @property
    def standard(self):
        return self.event_key.split('|')[1]

    @property
    def key(self):
        return self.event_key.split('|')[2]


@register_event('user_authorize_invoice')
class UserAuthorizeInvoiceEvent(BaseEvent):
    """
    用户授权发票事件
    （会包含一个订单号，不成功就失败）

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1497082828_r1cI2
    """
    event = 'user_authorize_invoice'
    success_order_id = StringField('SuccOrderId')  # 授权成功的订单号
    fail_order_id = StringField('FailOrderId')  # 授权失败的订单号
    app_id = StringField('AppId')  # 用于接收事件推送的公众号的AppId
    auth_source = StringField('Source')  # 授权来源，web表示来自微信内H5，app标识来自app


@register_event('update_invoice_status')
class UpdateInvoiceStatusEvent(BaseEvent):
    """
    发票状态更新事件

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1497082828_r1cI2
    """
    event = 'update_invoice_status'
    status = StringField('Status')  # 发票报销状态
    card_id = StringField('CardId')  # 发票 Card ID
    code = StringField('Code')  # 发票 Code


@register_event('submit_invoice_title')
class SubmitInvoiceTitleEvent(BaseEvent):
    """
    用户提交发票抬头事件

    详情请参考
    https://mp.weixin.qq.com/wiki?id=mp1496554912_vfWU0
    """
    event = 'submit_invoice_title'
    title = StringField('title')  # 抬头
    phone = StringField('phone')  # 联系方式
    tax_no = StringField('tax_no')  # 税号
    addr = StringField('addr')  # 地址
    bank_type = StringField('bank_type')  # 银行类型
    bank_no = StringField('bank_no')  # 银行号码
    attach = StringField('attach')  # 附加字段
    title_type = StringField('title_type')  # 抬头类型，个人InvoiceUserTitlePersonType, 公司InvoiceUserTitleBusinessType


@register_event('user_enter_tempsession')
class UserEnterTempSessionEvent(BaseEvent):
    """
    小程序用户进入客服消息
    详情请参阅
    https://developers.weixin.qq.com/miniprogram/dev/framework/open-ability/customer-message/receive.html
    """
    event = 'user_enter_tempsession'
    session_from = StringField('SessionFrom')


@register_event('view_miniprogram')
class ViewMiniProgramEvent(BaseEvent):
    """
    从菜单进入小程序事件
    """
    event = 'view_miniprogram'
    page_path = StringField('EventKey')  # 小程序路径
    menu_id = StringField('MenuId')  # 菜单ID


@register_event('wxa_media_check')
class WxaMediaCheckEvent(BaseEvent):
    """
    异步检测结果通知事件
    详情请参考
    https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/sec-check/security.mediaCheckAsync.html
    """

    event = 'wxa_media_check'
    is_risky = IntegerField('isrisky')  # 检测结果，0：暂未检测到风险，1：风险
    extra_info_json = StringField('extra_info_json')  # 附加信息，默认为空
    trace_id = StringField('trace_id')  # 任务 id
    status_code = IntegerField('status_code')  # 默认为：0，4294966288(-1008)为链接无法下载

    @property
    def is_valid(self):
        return self.is_risky == 0 and self.status_code == 0



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from optionaldict import optionaldict

from wechatpyyyy.client.api.base import BaseWeChatAPI
from wechatpyyyy.utils import to_text


class WeChatBatch(BaseWeChatAPI):
    """
    https://work.weixin.qq.com/api/doc#90000/90135/90979

    异步批量接口用于大批量数据的处理，提交后接口即返回，企业微信会在后台继续执行任务。
    执行完成后，企业微信后台会通过任务事件通知企业获取结果。事件的内容是加密的，解密过程请参考 [消息的加解密处理][signure]，任务事件请参考异步任务完成事件推送。
    目前，仅为通讯录更新提供了异步批量接口
    """

    def sync_user(self, url, token, encoding_aes_key, media_id, to_invite=True):
        """
        增量更新成员

        https://work.weixin.qq.com/api/doc#90000/90135/90980

        :param url: 企业应用接收企业微信推送请求的访问协议和地址，支持http或https协议
        :param token: 用于生成签名
        :param encoding_aes_key: 用于消息体的加密，是AES密钥的Base64编码
        :param media_id: 上传的csv文件的media_id
        :param to_invite: 是否邀请新建的成员使用企业微信（将通过微信服务通知或短信或邮件下发邀请，每天自动下发一次，最多持续3个工作日），默认值为true。
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'batch/syncuser',
            data={
                'media_id': media_id,
                'to_invite': to_invite,
                'callback': {
                    'url': url,
                    'token': token,
                    'encodingaeskey': encoding_aes_key
                }
            }
        )

    def replace_user(self, url, token, encoding_aes_key, media_id, to_invite=True):
        """
        全量覆盖成员

        https://work.weixin.qq.com/api/doc#90000/90135/90981

        :param url: 企业应用接收企业微信推送请求的访问协议和地址，支持http或https协议
        :param token: 用于生成签名
        :param encoding_aes_key: 用于消息体的加密，是AES密钥的Base64编码
        :param media_id: 上传的csv文件的media_id
        :param to_invite: 是否邀请新建的成员使用企业微信（将通过微信服务通知或短信或邮件下发邀请，每天自动下发一次，最多持续3个工作日），默认值为true。
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'batch/replaceuser',
            data={
                'media_id': media_id,
                'to_invite': to_invite,
                'callback': {
                    'url': url,
                    'token': token,
                    'encodingaeskey': encoding_aes_key
                }
            }
        )

    def replace_party(self, url, token, encoding_aes_key, media_id):
        """
        全量覆盖部门

        https://work.weixin.qq.com/api/doc#90000/90135/90982

        :param url: 企业应用接收企业微信推送请求的访问协议和地址，支持http或https协议
        :param token: 用于生成签名
        :param encoding_aes_key: 用于消息体的加密，是AES密钥的Base64编码
        :param media_id: 上传的csv文件的media_id
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'batch/replaceparty',
            data={
                'media_id': media_id,
                'callback': {
                    'url': url,
                    'token': token,
                    'encodingaeskey': encoding_aes_key
                }
            }
        )

    def get_result(self, job_id):
        """
        获取异步任务结果

        https://work.weixin.qq.com/api/doc#90000/90135/90983

        :param job_id: 异步任务id，最大长度为64字符
        :return: 返回的 JSON 数据包
        """
        return self._get('batch/getresult', params={'jobid': job_id})

    def invite(self, user=None, party=None, tag=None):
        """
        邀请成员

        https://work.weixin.qq.com/api/doc#90000/90135/90975

        企业可通过接口批量邀请成员使用企业微信，邀请后将通过短信或邮件下发通知。

        :param user: 成员ID列表, 最多支持1000个。
        :param party: 成员ID列表, 最多支持100个。
        :param tag: 成员ID列表, 最多支持100个。
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(user=user, party=party, tag=tag)
        return self._post('batch/invite', data=data)

    def invite_user(self, url, token, encoding_aes_key, user_ids=None,
                    party_ids=None, tag_ids=None, invite_tips=None):
        """
        邀请成员关注(deprecated)
        https://qydev.weixin.qq.com/wiki/index.php?title=异步任务接口

        :param url: 企业应用接收企业微信推送请求的访问协议和地址，支持http或https协议
        :param token: 用于生成签名
        :param encoding_aes_key: 用于消息体的加密，是AES密钥的Base64编码
        :param user_ids: 可选，成员ID列表，多个接收者用‘|’分隔，最多支持1000个。
        :param party_ids: 可选，部门ID列表，多个接收者用‘|’分隔，最多支持100个。
        :param tag_ids: 可选，标签ID列表，多个接收者用‘|’分隔。
        :param invite_tips: 可选，推送到微信上的提示语
        :return: 返回的 JSON 数据包
        """
        data = optionaldict()
        data['callback'] = {
            'url': url,
            'token': token,
            'encodingaeskey': encoding_aes_key
        }
        if isinstance(user_ids, (tuple, list)):
            user_ids = '|'.join(map(to_text, user_ids))
        if isinstance(party_ids, (tuple, list)):
            party_ids = '|'.join(map(to_text, party_ids))
        if isinstance(tag_ids, (tuple, list)):
            tag_ids = '|'.join(map(to_text, tag_ids))
        data['touser'] = user_ids
        data['toparty'] = party_ids
        data['totag'] = tag_ids
        data['invite_tips'] = invite_tips
        return self._post('batch/inviteuser', data=data)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import random
from datetime import datetime

from wechatpyyyy.pay.base import BaseWeChatPayAPI


class WeChatCoupon(BaseWeChatPayAPI):

    def send(self, user_id, stock_id, op_user_id=None, device_info=None,
             out_trade_no=None):
        """
        发放代金券

        :param user_id: 用户在公众号下的 openid
        :param stock_id: 代金券批次 ID
        :param op_user_id: 可选，操作员账号，默认为商户号
        :param device_info: 可选，微信支付分配的终端设备号
        :param out_trade_no: 可选，商户订单号，需保持唯一性，默认自动生成
        :return: 返回的结果信息
        """
        if not out_trade_no:
            now = datetime.now()
            out_trade_no = '{0}{1}{2}'.format(
                self.mch_id,
                now.strftime('%Y%m%d%H%M%S'),
                random.randint(1000, 10000)
            )
        data = {
            'appid': self.appid,
            'coupon_stock_id': stock_id,
            'openid': user_id,
            'openid_count': 1,
            'partner_trade_no': out_trade_no,
            'op_user_id': op_user_id,
            'device_info': device_info,
            'version': '1.0',
            'type': 'XML',
        }
        return self._post('mmpaymkttransfers/send_coupon', data=data)

    def query_stock(self, stock_id, op_user_id=None, device_info=None):
        """
        查询代金券批次

        :param stock_id: 代金券批次 ID
        :param op_user_id: 可选，操作员账号，默认为商户号
        :param device_info: 可选，微信支付分配的终端设备号
        :return: 返回的结果信息
        """
        data = {
            'appid': self.appid,
            'coupon_stock_id': stock_id,
            'op_user_id': op_user_id,
            'device_info': device_info,
            'version': '1.0',
            'type': 'XML',
        }
        return self._post('mmpaymkttransfers/query_coupon_stock', data=data)

    def query_coupon(self, coupon_id, user_id,
                     op_user_id=None, device_info=None):
        """
        查询代金券信息

        :param coupon_id: 代金券 ID
        :param user_id: 用户在公众号下的 openid
        :param op_user_id: 可选，操作员账号，默认为商户号
        :param device_info: 可选，微信支付分配的终端设备号
        :return: 返回的结果信息
        """
        data = {
            'coupon_id': coupon_id,
            'openid': user_id,
            'appid': self.appid,
            'op_user_id': op_user_id,
            'device_info': device_info,
            'version': '1.0',
            'type': 'XML',
        }
        return self._post('promotion/query_coupon', data=data)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from wechatpy import messages
from wechatpyyyy.fields import IntegerField, StringField

MESSAGE_TYPES = {}


def register_message(msg_type):
    def register(cls):
        MESSAGE_TYPES[msg_type] = cls
        return cls

    return register


@register_message('text')
class TextMessage(messages.TextMessage):
    agent = IntegerField('AgentID', 0)


@register_message('image')
class ImageMessage(messages.ImageMessage):
    agent = IntegerField('AgentID', 0)


@register_message('voice')
class VoiceMessage(messages.VoiceMessage):
    agent = IntegerField('AgentID', 0)


@register_message('shortvideo')
class ShortVideoMessage(messages.ShortVideoMessage):
    agent = IntegerField('AgentID', 0)


@register_message('video')
class VideoMessage(messages.VideoMessage):
    agent = IntegerField('AgentID', 0)


@register_message('location')
class LocationMessage(messages.LocationMessage):
    agent = IntegerField('AgentID', 0)


@register_message('link')
class LinkMessage(messages.LinkMessage):
    agent = IntegerField('AgentID', 0)
    pic_url = StringField('PicUrl')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import json
import datetime

from optionaldict import optionaldict

from wechatpyyyy.client.api.base import BaseWeChatAPI


class WeChatMarketing(BaseWeChatAPI):
    API_BASE_URL = 'https://api.weixin.qq.com/marketing/'

    def add_user_action_sets(self, _type, name, description, version='v1.0'):
        """
        创建数据源
        https://wximg.qq.com/wxp/pdftool/get.html?id=rkalQXDBM&pa=39

        :param _type: 用户行为源类型
        :param name: 用户行为源名称 必填
        :param description: 用户行为源描述，字段长度最小 1 字节，长度最大 128 字节
        :param version: 版本号 v1.0
        :return: 数据源唯一ID
        """
        return self._post(
            'user_action_sets/add',
            params={'version': version},
            json=optionaldict(
                type=_type,
                name=name,
                description=description,
                version=version
            ),
            result_processor=lambda x: x['data']['user_action_set_id']
        )

    def get_user_action_sets(self, user_action_set_id, version='v1.0'):
        """
        获取数据源信息

        :param user_action_set_id: 数据源唯一ID
        :param version: 版本号 v1.0
        """
        return self._get(
            'user_action_sets/get',
            params={'version': version, 'user_action_set_id': user_action_set_id},
            result_processor=lambda x: x['data']['list']
        )

    def add_user_actions(self, actions=(), version='v1.0'):
        """
        回传数据

        https://wximg.qq.com/wxp/pdftool/get.html?id=rkalQXDBM&pa=39

        :param actions: 用户行为源类型
        :param version: 版本号 v1.0
        """
        return self._post(
            'user_actions/add',
            params={'version': version},
            json={'actions': actions}
        )

    def get_ad_leads(self, start_date=None, end_date=None, filtering=(), page=1, page_size=100, version='v1.0'):
        """
         获取朋友圈销售线索数据接口

        :param start_date: 开始日期 默认今天
        :param end_date: 结束日期 默认今天
        :param filtering: 过滤条件 [{field: 过滤字段, operator: 操作符, values: 字段取值}]
        :param page: 页码，获取指定页数据
        :param page_size: 一页获取的数据条数(1-100)
        :param version: 版本号 v1.0
        """
        today = datetime.date.today()
        if start_date is None:
            start_date = today
        if end_date is None:
            end_date = today
        if isinstance(start_date, datetime.date):
            start_date = start_date.strftime("%Y-%m-%d")
        if isinstance(end_date, datetime.date):
            end_date = end_date.strftime("%Y-%m-%d")

        return self._get(
            'wechat_ad_leads/get',
            params=optionaldict(
                date_range=json.dumps({'start_date': start_date, 'end_date': end_date}),
                filtering=json.dumps(filtering) if filtering else None,
                page=page,
                page_size=page_size,
                version=version
            ),
            result_processor=lambda x: x['data']
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from wechatpyyyy.client.api.base import BaseWeChatAPI


class WeChatInvoice(BaseWeChatAPI):
    API_BASE_URL = 'https://api.weixin.qq.com/card/invoice/'

    def get_url(self):
        """
        获取自身开票平台专用的授权链接
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1496561481_1TyO7

        :return:该开票平台专用的授权链接
        """
        return self._post(
            'seturl',
            data={},
            result_processor=lambda x: x['invoice_url'],
        )

    def create_card(self, base_info, payee, invoice_type, detail=None):
        """
        创建发票卡券模板
        注意这里的对象和会员卡有类似之处，但是含义有不同。创建发票卡券模板是创建发票卡券的基础。
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1496561481_1TyO7

        :param base_info:发票卡券模板基础信息
        :type base_info: dict
        :param payee: 收款方（开票方）全称，显示在发票详情内。建议一个收款方对应一个发票卡券模板
        :param invoice_type: 发票类型描述
        :param detail: 备注详情
        :return: 发票卡券模板的编号，用于后续该商户发票生成后，作为必填参数在调用插卡接口时传入
        """
        return self._post(
            'platform/createcard',
            data={
                'invoice_info': {
                    'base_info': base_info,
                    'payee': payee,
                    'type': invoice_type,
                    'detail': detail,
                },
            },
            result_processor=lambda x: x['card_id'],
        )

    def get_auth_url(self, s_pappid, order_id, money, timestamp, source, ticket, auth_type, redirect_url=None):
        """
        获取授权页链接
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1497082828_r1cI2

        :param s_pappid: 开票平台在微信的标识号，商户需要找开票平台提供
        :param order_id: 订单id，在商户内单笔开票请求的唯一识别号
        :param money: 订单金额，以分为单位
        :type money: int
        :param timestamp: Unix 时间戳
        :type timestamp: int
        :param source: 开票来源。app: App开票, web: 微信H5开票, wap: 普通网页开票
        :param ticket: 根据获取授权ticket接口取得
        :param auth_type: 授权类型。0: 开票授权，1: 填写字段开票授权，2: 领票授权
        :type auth_type: int
        :param redirect_url: 授权成功后跳转页面。本字段只有在source为H5的时候需要填写。
        :return: 获取授权页链接
        """
        if source not in {'app', 'web', 'wap'}:
            raise ValueError('Unsupported source. Valid sources are "app", "web" or "wap"')
        if source == 'web' and redirect_url is None:
            raise ValueError('redirect_url is required if source is web')
        if not (0 <= auth_type <= 2):
            raise ValueError('Unsupported auth type. Valid auth types are 0, 1 or 2')
        return self._post(
            'getauthurl',
            data={
                's_pappid': s_pappid,
                'order_id': order_id,
                'money': money,
                'timestamp': timestamp,
                'source': source,
                'ticket': ticket,
                'type': auth_type,
                'redirect_url': redirect_url,
            },
            result_processor=lambda x: x['auth_url'],
        )

    def set_auth_field(self, user_field, biz_field):
        """
        设置授权页字段信息
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1497082828_r1cI2

        :param user_field: 授权页个人发票字段
        :type user_field: dict
        :param biz_field: 授权页单位发票字段
        :type biz_field: dict
        """
        return self._post(
            'setbizattr',
            params={
                'action': 'set_auth_field',
            },
            data={
                'auth_field': {
                    'user_field': user_field,
                    'biz_field': biz_field,
                },
            },
        )

    def get_auth_field(self):
        """
        获取授权页字段信息
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1497082828_r1cI2

        :return: 授权页的字段设置
        :rtype: dict
        """
        return self._post(
            'setbizattr',
            params={
                'action': 'get_auth_field',
            },
            data={},
        )

    def get_auth_data(self, s_pappid, order_id):
        """
        查询授权数据
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1497082828_r1cI2

        :param s_pappid: 开票平台在微信的标识号，商户需要找开票平台提供
        :param order_id: 订单id，在商户内单笔开票请求的唯一识别号
        :return: 用户的开票信息
        :rtype: dict
        """
        return self._post(
            'getauthdata',
            data={
                's_pappid': s_pappid,
                'order_id': order_id,
            },
        )

    def reject_insert(self, s_pappid, order_id, reason, redirect_url=None):
        """
        拒绝用户的开发票请求
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1497082828_r1cI2

        :param s_pappid: 开票平台在微信的标识号，商户需要找开票平台提供
        :param order_id: 订单id，在商户内单笔开票请求的唯一识别号
        :param reason: 拒绝原因
        :param redirect_url: 跳转链接
        """
        return self._post(
            'rejectinsert',
            data={
                's_pappid': s_pappid,
                'order_id': order_id,
                'reason': reason,
                'url': redirect_url,
            },
        )

    def insert(self, order_id, card_id, appid, card_ext):
        """
        制作发票卡券，并放入用户卡包
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1497082828_r1cI2

        :param order_id: 订单id，在商户内单笔开票请求的唯一识别号
        :param card_id: 发票卡券模板的编号
        :param appid: 商户 AppID
        :param card_ext: 发票具体内容
        :type card_ext: dict
        :return: 随机防重字符串，以及用户 Open ID
        """
        return self._post(
            'insert',
            data={
                'order_id': order_id,
                'card_id': card_id,
                'appid': appid,
                'card_ext': card_ext,
            },
        )

    def upload_pdf(self, pdf):
        """
        上传电子发票中的消费凭证 PDF
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1497082828_r1cI2

        :param pdf: 要上传的 PDF 文件，一个 File-object
        :return: 64位整数，在将发票卡券插入用户卡包时使用用于关联pdf和发票卡券。有效期为3天。
        """
        return self._post(
            'platform/setpdf',
            files={
                'pdf': pdf,
            },
            result_processor=lambda x: x['s_media_id'],
        )

    def get_pdf(self, s_media_id):
        """
        查询已上传的 PDF
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1497082828_r1cI2

        :param s_media_id: PDF 文件上传时的 s_media_id
        :return: PDF 文件的 URL，以及过期时间
        :rtype: dict
        """
        return self._post(
            'platform/getpdf',
            params={
                'action': 'get_url',
            },
            data={
                's_media_id': s_media_id,
            },
        )

    def update_status(self, card_id, code, reimburse_status):
        """
        更新发票卡券的状态
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1497082828_r1cI2

        :param card_id: 发票卡券模板的编号
        :param code: 发票卡券的编号
        :param reimburse_status: 发票报销状态
        """
        return self._post(
            'platform/updatestatus',
            data={
                'card_id': card_id,
                'code': code,
                'reimburse_status': reimburse_status,
            },
        )

    def set_pay_mch(self, mchid, s_pappid):
        """
        关联商户号与开票平台，设置支付后开票
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1496561731_2Z55U

        :param mchid: 微信支付商户号
        :param s_pappid: 开票平台在微信的标识号，商户需要找开票平台提供
        """
        return self._post(
            'setbizattr',
            params={
                'action': 'set_pay_mch',
            },
            data={
                'paymch_info': {
                    'mchid': mchid,
                    's_pappid': s_pappid,
                },
            },
        )

    def get_pay_mch(self):
        """
        查询商户号与开票平台关联情况
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1496561731_2Z55U

        :return: mchid 和 s_pappid
        :rtype: dict
        """
        return self._post(
            'setbizattr',
            params={
                'action': 'get_pay_mch',
            },
            data={},
        )

    def get_reimburse(self, card_id, encrypt_code):
        """
        报销方查询发票信息
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1496561749_f7T6D

        :param card_id: 发票卡券的 Card ID
        :param encrypt_code: 发票卡券的加密 Code
        :return: 电子发票的结构化信息
        :rtype: dict
        """
        return self._post(
            'reimburse/getinvoiceinfo',
            data={
                'card_id': card_id,
                'encrypt_code': encrypt_code,
            },
        )

    def update_reimburse(self, card_id, encrypt_code, reimburse_status):
        """
        报销方更新发票信息
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1496561749_f7T6D

        :param card_id: 发票卡券的 Card ID
        :param encrypt_code: 发票卡券的加密 Code
        :param reimburse_status: 发票报销状态
        """
        return self._post(
            'reimburse/updateinvoicestatus',
            data={
                'card_id': card_id,
                'encrypt_code': encrypt_code,
                'reimburse_status': reimburse_status,
            },
        )

    def batch_update_reimburse(self, openid, reimburse_status, invoice_list):
        """
        报销方批量更新发票信息
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1496561749_f7T6D

        :param openid: 用户的 Open ID
        :param reimburse_status: 发票报销状态
        :param invoice_list: 发票列表
        :type invoice_list: list[dict]
        """
        return self._post(
            'reimburse/updatestatusbatch',
            data={
                'openid': openid,
                'reimburse_status': reimburse_status,
                'invoice_list': invoice_list,
            },
        )

    def get_user_title_url(
            self, user_fill, title=None, phone=None, tax_no=None, addr=None, bank_type=None, bank_no=None,
            out_title_id=None):
        """
        获取添加发票链接
        获取链接，发送给用户。用户同意以后，发票抬头信息将会录入到用户微信中
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1496554912_vfWU0

        :param user_fill: 企业设置抬头为0，用户自己填写抬头为1
        :type user_fill: bool
        :param title: 抬头，当 user_fill 为 False 时必填
        :param phone: 联系方式
        :param tax_no: 税号
        :param addr: 地址
        :param bank_type: 银行类型
        :param bank_no: 银行号码
        :param out_title_id: 开票码
        :return: 添加发票的链接
        """
        if user_fill and title is None:
            raise ValueError('title is required when user_fill is False')
        return self._post(
            'biz/getusertitleurl',
            data={
                'user_fill': int(user_fill),
                'title': title,
                'phone': phone,
                'tax_no': tax_no,
                'addr': addr,
                'bank_type': bank_type,
                'bank_no': bank_no,
                'out_title_id': out_title_id,
            },
            result_processor=lambda x: x['url'],
        )

    def get_select_title_url(self, attach=None):
        """
        获取商户专属开票链接
        商户调用接口，获取链接。用户扫码，可以选择抬头发给商户。可以将链接转成二维码，立在收银台。
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1496554912_vfWU0

        :param attach: 附加字段，用户提交发票时会发送给商户
        :return: 商户专属开票链接
        """
        return self._post(
            'biz/getselecttitleurl',
            data={
                'attach': attach,
            },
            result_processor=lambda x: x['url'],
        )

    def scan_title(self, scan_text):
        """
        根据扫描码，获取用户发票抬头
        商户扫用户“我的—个人信息—我的发票抬头”里面的抬头二维码后，通过调用本接口，可以获取用户抬头信息
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1496554912_vfWU0

        :param scan_text: 扫码后获取的文本
        :return: 用户的发票抬头数据
        :rtype: dict
        """
        return self._post(
            'scantitle',
            data={
                'scan_text': scan_text,
            },
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from wechatpyyyy.client.api.base import BaseWeChatAPI


class WeChatTemplate(BaseWeChatAPI):

    def set_industry(self, industry_id1, industry_id2):
        """
        设置所属行业
        详情请参考
        https://mp.weixin.qq.com/wiki?t=resource/res_main&id=mp1433751277

        :param industry_id1: 公众号模板消息所属行业编号
        :param industry_id2: 公众号模板消息所属行业编号
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'template/api_set_industry',
            data={
                'industry_id1': industry_id1,
                'industry_id2': industry_id2
            }
        )

    def get_industry(self):
        """
        获取设置的行业信息
        详情请参考
        https://mp.weixin.qq.com/wiki?t=resource/res_main&id=mp1433751277

        :return: 返回的 JSON 数据包
        """
        return self._get(
            'template/get_industry'
        )

    def get(self, template_id_short):
        """
        获得模板ID
        详情请参考
        https://mp.weixin.qq.com/wiki?t=resource/res_main&id=mp1433751277

        :param template_id_short: 模板库中模板的编号，有“TM**”和“OPENTMTM**”等形式
        :return: 模板 ID
        """
        res = self._post(
            'template/api_add_template',
            data={
                'template_id_short': template_id_short
            },
            result_processor=lambda x: x['template_id']
        )
        return res

    add = get

    def get_all_private_template(self):
        """
        获取模板列表
        详情请参考
        https://mp.weixin.qq.com/wiki?t=resource/res_main&id=mp1433751277

        :return: 返回的 JSON 数据包
        """
        return self._get(
            'template/get_all_private_template'
        )

    def del_private_template(self, template_id):
        """
        删除模板
        详情请参考
        https://mp.weixin.qq.com/wiki?t=resource/res_main&id=mp1433751277

        :param template_id: 公众帐号下模板消息ID
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'template/del_private_template',
            data={
                'template_id': template_id
            }
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from datetime import datetime, date

from wechatpyyyy.pay.base import BaseWeChatPayAPI


class WeChatTools(BaseWeChatPayAPI):

    def short_url(self, long_url):
        """
        长链接转短链接

        :param long_url: 长链接
        :return: 返回的结果数据
        """
        data = {
            'appid': self.appid,
            'long_url': long_url,
        }
        return self._post('tools/shorturl', data=data)

    def download_bill(self, bill_date, bill_type='ALL', device_info=None):
        """
        下载对账单

        :param bill_date: 下载对账单的日期
        :param bill_type: 账单类型，ALL，返回当日所有订单信息，默认值
                          SUCCESS，返回当日成功支付的订单,
                          REFUND，返回当日退款订单,
                          REVOKED，已撤销的订单
        :param device_info: 微信支付分配的终端设备号，填写此字段，只下载该设备号的对账单
        :return: 返回的结果数据
        """
        if isinstance(bill_date, (datetime, date)):
            bill_date = bill_date.strftime('%Y%m%d')

        data = {
            'appid': self.appid,
            'bill_date': bill_date,
            'bill_type': bill_type,
            'device_info': device_info,
        }
        return self._post('pay/downloadbill', data=data)

    def download_fundflow(self, bill_date, account_type='Basic',
                          tar_type=None):
        """
        下载资金账单
        https://pay.weixin.qq.com/wiki/doc/api/jsapi.php?chapter=9_18&index=7

        :param bill_date: 下载对账单的日期
        :param account_type: 账单的资金来源账户
                             Basic  基本账户
                             Operation 运营账户
                             Fees 手续费账户
        :param tar_type: 非必传参数，固定值：GZIP，返回格式为.gzip的压缩包账单。
                         不传则默认为数据流形式。
        """
        if isinstance(bill_date, (datetime, date)):
            bill_date = bill_date.strftime('%Y%m%d')

        data = {
            'appid': self.appid,
            'bill_date': bill_date,
            'account_type': account_type,
            'sign_type': 'HMAC-SHA256'
        }
        if tar_type is not None:
            data['tar_type'] = tar_type
        return self._post('pay/downloadfundflow', data=data)

    def auto_code_to_openid(self, auth_code):
        """
        授权码查询 openid 接口

        :param auth_code: 扫码支付授权码，设备读取用户微信中的条码或者二维码信息
        :return: 返回的结果数据
        """
        data = {
            'appid': self.appid,
            'auth_code': auth_code,
        }
        return self._post('tools/authcodetoopenid', data=data)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from wechatpyyyy.utils import to_text
from wechatpyyyy.client.api.base import BaseWeChatAPI


class WeChatGroup(BaseWeChatAPI):

    def create(self, name):
        """
        创建分组

        详情请参考
        http://mp.weixin.qq.com/wiki/0/56d992c605a97245eb7e617854b169fc.html

        :param name: 分组名字（30个字符以内）
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.group.create('New Group')

        """
        name = to_text(name)
        return self._post(
            'groups/create',
            data={'group': {'name': name}}
        )

    def get(self, user_id=None):
        """
        查询所有分组或查询用户所在分组 ID

        详情请参考
        http://mp.weixin.qq.com/wiki/0/56d992c605a97245eb7e617854b169fc.html

        :param user_id: 用户 ID，提供时查询该用户所在分组，否则查询所有分组
        :return: 所有分组列表或用户所在分组 ID

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            group = client.group.get('openid')

        """
        if user_id is None:
            res = self._get(
                'groups/get',
                result_processor=lambda x: x['groups']
            )
        else:
            res = self._post(
                'groups/getid',
                data={'openid': user_id},
                result_processor=lambda x: x['groupid']
            )
        return res

    def update(self, group_id, name):
        """
        修改分组名

        详情请参考
        http://mp.weixin.qq.com/wiki/0/56d992c605a97245eb7e617854b169fc.html

        :param group_id: 分组id，由微信分配
        :param name: 分组名字（30个字符以内）
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.group.update(1234, 'New Name')

        """
        name = to_text(name)
        return self._post(
            'groups/update',
            data={
                'group': {
                    'id': int(group_id),
                    'name': name
                }
            }
        )

    def move_user(self, user_id, group_id):
        """
        移动用户分组

        详情请参考
        http://mp.weixin.qq.com/wiki/0/56d992c605a97245eb7e617854b169fc.html

        :param user_id: 用户 ID, 可以是单个或者列表，为列表时为批量移动用户分组
        :param group_id: 分组 ID
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.group.move_user('openid', 1234)

        """
        data = {'to_groupid': group_id}
        if isinstance(user_id, (tuple, list)):
            endpoint = 'groups/members/batchupdate'
            data['openid_list'] = user_id
        else:
            endpoint = 'groups/members/update'
            data['openid'] = user_id
        return self._post(endpoint, data=data)

    def delete(self, group_id):
        """
        删除分组

        详情请参考
        http://mp.weixin.qq.com/wiki/0/56d992c605a97245eb7e617854b169fc.html

        :param group_id: 分组 ID
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.group.delete(1234)

        """
        return self._post(
            'groups/delete',
            data={
                'group': {
                    'id': group_id
                }
            }
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import struct
import socket
import base64

from wechatpyyyy.utils import to_text, to_binary, random_string, byte2int
from wechatpyyyy.crypto.pkcs7 import PKCS7Encoder
try:
    from wechatpyyyy.crypto.cryptography import WeChatCipher, AesEcbCipher
except ImportError:
    try:
        from wechatpyyyy.crypto.pycrypto import WeChatCipher, AesEcbCipher
    except ImportError:
        raise Exception('You must install either cryptography or pycryptodome!')


class BasePrpCrypto(object):

    def __init__(self, key):
        self.cipher = WeChatCipher(key)

    def get_random_string(self):
        return random_string(16)

    def _encrypt(self, text, _id):
        text = to_binary(text)
        tmp_list = []
        tmp_list.append(to_binary(self.get_random_string()))
        length = struct.pack(b'I', socket.htonl(len(text)))
        tmp_list.append(length)
        tmp_list.append(text)
        tmp_list.append(to_binary(_id))

        text = b''.join(tmp_list)
        text = PKCS7Encoder.encode(text)

        ciphertext = to_binary(self.cipher.encrypt(text))
        return base64.b64encode(ciphertext)

    def _decrypt(self, text, _id, exception=None):
        text = to_binary(text)
        plain_text = self.cipher.decrypt(base64.b64decode(text))
        padding = byte2int(plain_text[-1])
        content = plain_text[16:-padding]
        xml_length = socket.ntohl(struct.unpack(b'I', content[:4])[0])
        xml_content = to_text(content[4:xml_length + 4])
        from_id = to_text(content[xml_length + 4:])
        if from_id != _id:
            exception = exception or Exception
            raise exception()
        return xml_content


class BaseRefundCrypto(object):

    def __init__(self, key):
        self.cipher = AesEcbCipher(key)

    def _encrypt(self, text):
        text = to_binary(text)
        text = PKCS7Encoder.encode(text)

        ciphertext = to_binary(self.cipher.encrypt(text))
        return base64.b64encode(ciphertext)

    def _decrypt(self, text, exception=None):
        text = to_binary(text)
        plain_text = self.cipher.decrypt(base64.b64decode(text))
        padding = byte2int(plain_text[-1])
        content = plain_text[:-padding]
        return content



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from optionaldict import optionaldict
from wechatpyyyy.client.api.base import BaseWeChatAPI


class MerchantOrder(BaseWeChatAPI):

    API_BASE_URL = 'https://api.weixin.qq.com/'

    def get(self, order_id):
        res = self._post(
            'merchant/order/getbyid',
            data={
                'order_id': order_id
            },
            result_processor=lambda x: x['order']
        )
        return res

    def get_by_filter(self, status=None, begin_time=None, end_time=None):
        filter_dict = optionaldict(
            status=status,
            begintime=begin_time,
            endtime=end_time
        )

        res = self._post(
            'merchant/order/getbyfilter',
            data=dict(filter_dict),
            result_processor=lambda x: x['order_list']
        )
        return res

    def set_delivery(self, order_id, company, track_no,
                     need_delivery=1, is_others=0):
        return self._post(
            'merchant/order/setdelivery',
            data={
                'order_id': order_id,
                'delivery_company': company,
                'delivery_track_no': track_no,
                'need_delivery': need_delivery,
                'is_others': is_others
            }
        )

    def close(self, order_id):
        return self._post(
            'merchant/order/close',
            data={
                'order_id': order_id
            }
        )



# -*- coding: utf-8 -*-
"""
    wechatpyyyy.oauth
    ~~~~~~~~~~~~~~~

    This module provides OAuth2 library for WeChat

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""
from __future__ import absolute_import, unicode_literals

import requests
from six.moves.urllib.parse import quote

from wechatpyyyy.exceptions import WeChatOAuthException
from wechatpyyyy.utils import json


class WeChatOAuth(object):
    """ 微信公众平台 OAuth 网页授权

    详情请参考
    https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&t=resource/res_list&verify=1&id=open1419316505
    """

    API_BASE_URL = 'https://api.weixin.qq.com/'
    OAUTH_BASE_URL = 'https://open.weixin.qq.com/connect/'

    def __init__(self, app_id, secret, redirect_uri, scope='snsapi_base', state=''):
        """

        :param app_id: 微信公众号 app_id
        :param secret: 微信公众号 secret
        :param redirect_uri: OAuth2 redirect URI
        :param scope: 可选，微信公众号 OAuth2 scope，默认为 ``snsapi_base``
        :param state: 可选，微信公众号 OAuth2 state
        """
        self.app_id = app_id
        self.secret = secret
        self.redirect_uri = redirect_uri
        self.scope = scope
        self.state = state
        self._http = requests.Session()

    def _request(self, method, url_or_endpoint, **kwargs):
        if not url_or_endpoint.startswith(('http://', 'https://')):
            url = '{base}{endpoint}'.format(
                base=self.API_BASE_URL,
                endpoint=url_or_endpoint
            )
        else:
            url = url_or_endpoint

        if isinstance(kwargs.get('data', ''), dict):
            body = json.dumps(kwargs['data'], ensure_ascii=False)
            body = body.encode('utf-8')
            kwargs['data'] = body

        res = self._http.request(
            method=method,
            url=url,
            **kwargs
        )
        try:
            res.raise_for_status()
        except requests.RequestException as reqe:
            raise WeChatOAuthException(
                errcode=None,
                errmsg=None,
                client=self,
                request=reqe.request,
                response=reqe.response
            )
        result = json.loads(res.content.decode('utf-8', 'ignore'), strict=False)

        if 'errcode' in result and result['errcode'] != 0:
            errcode = result['errcode']
            errmsg = result['errmsg']
            raise WeChatOAuthException(
                errcode,
                errmsg,
                client=self,
                request=res.request,
                response=res
            )

        return result

    def _get(self, url, **kwargs):
        return self._request(
            method='get',
            url_or_endpoint=url,
            **kwargs
        )

    @property
    def authorize_url(self):
        """获取授权跳转地址

        :return: URL 地址
        """
        redirect_uri = quote(self.redirect_uri, safe=b'')
        url_list = [
            self.OAUTH_BASE_URL,
            'oauth2/authorize?appid=',
            self.app_id,
            '&redirect_uri=',
            redirect_uri,
            '&response_type=code&scope=',
            self.scope
        ]
        if self.state:
            url_list.extend(['&state=', self.state])
        url_list.append('#wechat_redirect')
        return ''.join(url_list)

    @property
    def qrconnect_url(self):
        """生成扫码登录地址

        :return: URL 地址
        """
        redirect_uri = quote(self.redirect_uri, safe=b'')
        url_list = [
            self.OAUTH_BASE_URL,
            'qrconnect?appid=',
            self.app_id,
            '&redirect_uri=',
            redirect_uri,
            '&response_type=code&scope=',
            'snsapi_login'  # scope
        ]
        if self.state:
            url_list.extend(['&state=', self.state])
        url_list.append('#wechat_redirect')
        return ''.join(url_list)

    def fetch_access_token(self, code):
        """获取 access_token

        :param code: 授权完成跳转回来后 URL 中的 code 参数
        :return: JSON 数据包
        """
        res = self._get(
            'sns/oauth2/access_token',
            params={
                'appid': self.app_id,
                'secret': self.secret,
                'code': code,
                'grant_type': 'authorization_code'
            }
        )
        self.access_token = res['access_token']
        self.open_id = res['openid']
        self.refresh_token = res['refresh_token']
        self.expires_in = res['expires_in']
        return res

    def refresh_access_token(self, refresh_token):
        """刷新 access token

        :param refresh_token: OAuth2 refresh token
        :return: JSON 数据包
        """
        res = self._get(
            'sns/oauth2/refresh_token',
            params={
                'appid': self.app_id,
                'grant_type': 'refresh_token',
                'refresh_token': refresh_token
            }
        )
        self.access_token = res['access_token']
        self.open_id = res['openid']
        self.refresh_token = res['refresh_token']
        self.expires_in = res['expires_in']
        return res

    def get_user_info(self, openid=None, access_token=None, lang='zh_CN'):
        """获取用户信息

        :param openid: 可选，微信 openid，默认获取当前授权用户信息
        :param access_token: 可选，access_token，默认使用当前授权用户的 access_token
        :param lang: 可选，语言偏好, 默认为 ``zh_CN``
        :return: JSON 数据包
        """
        openid = openid or self.open_id
        access_token = access_token or self.access_token
        return self._get(
            'sns/userinfo',
            params={
                'access_token': access_token,
                'openid': openid,
                'lang': lang
            }
        )

    def check_access_token(self, openid=None, access_token=None):
        """检查 access_token 有效性

        :param openid: 可选，微信 openid，默认获取当前授权用户信息
        :param access_token: 可选，access_token，默认使用当前授权用户的 access_token
        :return: 有效返回 True，否则 False
        """
        openid = openid or self.open_id
        access_token = access_token or self.access_token
        res = self._get(
            'sns/auth',
            params={
                'access_token': access_token,
                'openid': openid
            }
        )
        if res['errcode'] == 0:
            return True
        return False



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import xmltodict

from wechatpyyyy.enterprise.events import EVENT_TYPES
from wechatpyyyy.enterprise.messages import MESSAGE_TYPES
from wechatpyyyy.messages import UnknownMessage
from wechatpyyyy.utils import to_text


def parse_message(xml):
    if not xml:
        return
    message = xmltodict.parse(to_text(xml))['xml']
    message_type = message['MsgType'].lower()
    if message_type == 'event':
        event_type = message['Event'].lower()
        message_class = EVENT_TYPES.get(event_type, UnknownMessage)
    else:
        message_class = MESSAGE_TYPES.get(message_type, UnknownMessage)
    return message_class(message)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from wechatpyyyy.client.api.base import BaseWeChatAPI


class WeChatMedia(BaseWeChatAPI):
    """ 素材管理

    https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/New_temporary_materials.html
    """

    def upload(self, media_type, media_file):
        """
        新增临时素材
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/New_temporary_materials.html

        :param media_type: 媒体文件类型，分别有图片（image）、语音（voice）、视频（video）和缩略图（thumb）
        :param media_file: 要上传的文件，一个 File-object

        :return: 返回的 JSON 数据包
        """
        return self._post(url='media/upload', params={'type': media_type}, files={'media': media_file})

    def download(self, media_id):
        """
        获取临时素材
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/Get_temporary_materials.html

        :param media_id: 媒体文件 ID

        :return: requests 的 Response 实例
        """
        return self._get('media/get', params={'media_id': media_id})

    def get_url(self, media_id):
        """
        获取临时素材下载地址

        :param media_id: 媒体文件 ID
        :return: 临时素材下载地址
        """
        return 'https://api.weixin.qq.com/cgi-bin/media/get' \
               '?access_token={}&media_id={}'.format(self.access_token, media_id)

    def upload_video(self, media_id, title, description):
        """
        群发视频消息时获取视频 media_id
        详情请参考
        http://mp.weixin.qq.com/wiki/15/5380a4e6f02f2ffdc7981a8ed7a40753.html

        :param media_id: 需通过基础支持中的上传下载多媒体文件 :func:`upload` 来得到
        :param title: 视频标题
        :param description: 视频描述

        :return: 返回的 JSON 数据包
        """
        return self._post(
            url='media/uploadvideo',
            data={
                'media_id': media_id,
                'title': title,
                'description': description
            }
        )

    def upload_articles(self, articles):
        """
        上传图文消息素材
        详情请参考
        http://mp.weixin.qq.com/wiki/15/5380a4e6f02f2ffdc7981a8ed7a40753.html

        :param articles: 图文消息数组
        :return: 返回的 JSON 数据包
        """
        articles_data = []
        for article in articles:
            articles_data.append({
                'thumb_media_id': article['thumb_media_id'],
                'title': article['title'],
                'content': article['content'],
                'author': article.get('author', ''),
                'content_source_url': article.get('content_source_url', ''),
                'digest': article.get('digest', ''),
                'show_cover_pic': article.get('show_cover_pic', 0)
            })
        return self._post(
            'media/uploadnews',
            data={
                'articles': articles_data
            }
        )

    def upload_image(self, media_file):
        """
        上传群发消息内的图片
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/Adding_Permanent_Assets.html

        :param media_file: 要上传的文件，一个 File-object
        :return: 上传成功时返回图片 URL
        """
        res = self._post(
            url='media/uploadimg',
            files={
                'media': media_file
            },
            result_processor=lambda x: x['url']
        )
        return res

    upload_mass_image = upload_image



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from optionaldict import optionaldict

from wechatpyyyy.client.api.base import BaseWeChatAPI


class WeChatUser(BaseWeChatAPI):
    """
    成员管理

    https://work.weixin.qq.com/api/doc#90000/90135/90194

    邀请成员接口位于 `WeChatBatch.invite`
    """

    def create(self, user_id, name, department=None, position=None,
               mobile=None, gender=0, tel=None, email=None,
               weixin_id=None, extattr=None, **kwargs):
        """
        创建成员

        https://work.weixin.qq.com/api/doc#90000/90135/90195
        """
        user_data = optionaldict()
        user_data['userid'] = user_id
        user_data['name'] = name
        user_data['gender'] = gender
        user_data['department'] = department
        user_data['position'] = position
        user_data['mobile'] = mobile
        user_data['tel'] = tel
        user_data['email'] = email
        user_data['weixinid'] = weixin_id
        user_data['extattr'] = extattr
        user_data.update(kwargs)

        return self._post(
            'user/create',
            data=user_data
        )

    def get(self, user_id):
        """
        读取成员

        https://work.weixin.qq.com/api/doc#90000/90135/90196
        """
        return self._get(
            'user/get',
            params={
                'userid': user_id
            }
        )

    def update(self, user_id, name=None, department=None, position=None,
               mobile=None, gender=None, tel=None, email=None,
               weixin_id=None, enable=None, extattr=None, **kwargs):
        """
        更新成员

        https://work.weixin.qq.com/api/doc#90000/90135/90197
        """
        user_data = optionaldict()
        user_data['userid'] = user_id
        user_data['name'] = name
        user_data['gender'] = gender
        user_data['department'] = department
        user_data['position'] = position
        user_data['mobile'] = mobile
        user_data['tel'] = tel
        user_data['email'] = email
        user_data['weixinid'] = weixin_id
        user_data['extattr'] = extattr
        user_data['enable'] = enable
        user_data.update(kwargs)

        return self._post(
            'user/update',
            data=user_data
        )

    def delete(self, user_id):
        """
        删除成员

        https://work.weixin.qq.com/api/doc#90000/90135/90198
        """
        return self._get(
            'user/delete',
            params={
                'userid': user_id
            }
        )

    def batch_delete(self, user_ids):
        """
        批量删除成员

        https://work.weixin.qq.com/api/doc#90000/90135/90199
        """
        return self._post(
            'user/batchdelete',
            data={
                'useridlist': user_ids
            }
        )

    def list(self, department_id, fetch_child=False, status=0, simple=False):
        """
        批量获取部门成员 / 批量获取部门成员详情

        https://work.weixin.qq.com/api/doc#90000/90135/90200
        https://work.weixin.qq.com/api/doc#90000/90135/90201

        此接口和 `WeChatDepartment.get_users` 是同一个接口，区别为 simple 的默认值不同。
        """
        url = 'user/simplelist' if simple else 'user/list'
        res = self._get(
            url,
            params={
                'department_id': department_id,
                'fetch_child': 1 if fetch_child else 0,
                'status': status
            }
        )
        return res['userlist']

    def convert_to_openid(self, user_id, agent_id=None):
        """
        user_id 转成 openid

        https://work.weixin.qq.com/api/doc#90000/90135/90202

        :param user_id: 企业微信内的成员 ID
        :param agent_id: 可选，需要发送红包的应用ID，若只是使用微信支付和企业转账，则无需该参数
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            userid=user_id,
            agentid=agent_id
        )
        return self._post('user/convert_to_openid', data=data)

    def convert_to_user_id(self, openid):
        """
        openid 转成 user_id

        https://work.weixin.qq.com/api/doc#90000/90135/90202

        :param openid: 在使用微信支付、微信红包和企业转账之后，返回结果的openid
        :return: 该 openid 在企业微信中对应的成员 user_id
        """
        res = self._post('user/convert_to_userid', data={'openid': openid})
        return res['userid']

    def verify(self, user_id):
        """
        二次验证

        https://work.weixin.qq.com/api/doc#90000/90135/90203

        :param user_id: 成员UserID。对应管理端的帐号
        """
        return self._get(
            'user/authsucc',
            params={
                'userid': user_id
            }
        )

    def get_info(self, agent_id, code):
        return self._get(
            'user/getuserinfo',
            params={
                'agentid': agent_id,
                'code': code
            }
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from Crypto.Cipher import AES


class BaseWeChatCipher(object):

    def __init__(self, cipher):
        self.cipher = cipher

    def encrypt(self, plaintext):
        return self.cipher.encrypt(plaintext)

    def decrypt(self, ciphertext):
        return self.cipher.decrypt(ciphertext)


class WeChatCipher(BaseWeChatCipher):

    def __init__(self, key, iv=None):
        iv = iv or key[:16]
        super(WeChatCipher, self).__init__(AES.new(key, AES.MODE_CBC, iv))


class AesEcbCipher(BaseWeChatCipher):

    def __init__(self, key):
        super(AesEcbCipher, self).__init__(AES.new(key, AES.MODE_ECB))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import datetime

import six

from wechatpyyyy.client.api.base import BaseWeChatAPI


class WeChatDataCube(BaseWeChatAPI):

    API_BASE_URL = 'https://api.weixin.qq.com/datacube/'

    @classmethod
    def _to_date_str(cls, date):
        if isinstance(date, (datetime.datetime, datetime.date)):
            return date.strftime('%Y-%m-%d')
        elif isinstance(date, six.string_types):
            return date
        else:
            raise ValueError('Can not convert %s type to str', type(date))

    def get_user_summary(self, begin_date, end_date):
        """
        获取用户增减数据
        详情请参考
        http://mp.weixin.qq.com/wiki/3/ecfed6e1a0a03b5f35e5efac98e864b7.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getusersummary',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            }
        )
        return res['list']

    def get_user_cumulate(self, begin_date, end_date):
        """
        获取累计用户数据
        详情请参考
        http://mp.weixin.qq.com/wiki/3/ecfed6e1a0a03b5f35e5efac98e864b7.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getusercumulate',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res

    def get_interface_summary(self, begin_date, end_date):
        """
        获取接口分析数据
        详情请参考
        http://mp.weixin.qq.com/wiki/8/30ed81ae38cf4f977194bf1a5db73668.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getinterfacesummary',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res

    def get_interface_summary_hour(self, begin_date, end_date):
        """
        获取接口分析分时数据
        详情请参考
        http://mp.weixin.qq.com/wiki/8/30ed81ae38cf4f977194bf1a5db73668.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getinterfacesummaryhour',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res

    def get_article_summary(self, begin_date, end_date):
        """
        获取图文群发每日数据
        详情请参考
        http://mp.weixin.qq.com/wiki/8/c0453610fb5131d1fcb17b4e87c82050.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getarticlesummary',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res

    def get_article_total(self, begin_date, end_date):
        """
        获取图文群发总数据
        详情请参考
        http://mp.weixin.qq.com/wiki/8/c0453610fb5131d1fcb17b4e87c82050.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getarticletotal',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res

    def get_user_read(self, begin_date, end_date):
        """
        获取图文统计数据
        详情请参考
        http://mp.weixin.qq.com/wiki/8/c0453610fb5131d1fcb17b4e87c82050.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getuserread',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res

    def get_user_read_hour(self, begin_date, end_date):
        """
        获取图文分时统计数据
        详情请参考
        http://mp.weixin.qq.com/wiki/8/c0453610fb5131d1fcb17b4e87c82050.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getuserreadhour',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res

    def get_user_share(self, begin_date, end_date):
        """
        获取图文分享转发数据
        详情请参考
        http://mp.weixin.qq.com/wiki/8/c0453610fb5131d1fcb17b4e87c82050.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getusershare',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res

    def get_user_share_hour(self, begin_date, end_date):
        """
        获取图文分享转发分时数据
        详情请参考
        http://mp.weixin.qq.com/wiki/8/c0453610fb5131d1fcb17b4e87c82050.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getusersharehour',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res

    def get_upstream_msg(self, begin_date, end_date):
        """
        获取消息发送概况数据
        详情请参考
        http://mp.weixin.qq.com/wiki/12/32d42ad542f2e4fc8a8aa60e1bce9838.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getupstreammsg',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res

    def get_upstream_msg_hour(self, begin_date, end_date):
        """
        获取消息发送分时数据
        详情请参考
        http://mp.weixin.qq.com/wiki/12/32d42ad542f2e4fc8a8aa60e1bce9838.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getupstreammsghour',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res

    def get_upstream_msg_week(self, begin_date, end_date):
        """
        获取消息发送周数据
        详情请参考
        http://mp.weixin.qq.com/wiki/12/32d42ad542f2e4fc8a8aa60e1bce9838.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getupstreammsgweek',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res

    def get_upstream_msg_month(self, begin_date, end_date):
        """
        获取消息发送月数据
        详情请参考
        http://mp.weixin.qq.com/wiki/12/32d42ad542f2e4fc8a8aa60e1bce9838.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getupstreammsgmonth',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res

    def get_upstream_msg_dist(self, begin_date, end_date):
        """
        获取消息发送分布数据
        详情请参考
        http://mp.weixin.qq.com/wiki/12/32d42ad542f2e4fc8a8aa60e1bce9838.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getupstreammsgdist',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res

    def get_upstream_msg_dist_week(self, begin_date, end_date):
        """
        获取消息发送分布数据
        详情请参考
        http://mp.weixin.qq.com/wiki/12/32d42ad542f2e4fc8a8aa60e1bce9838.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getupstreammsgdistweek',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res

    def get_upstream_msg_dist_month(self, begin_date, end_date):
        """
        获取消息发送分布数据
        详情请参考
        http://mp.weixin.qq.com/wiki/12/32d42ad542f2e4fc8a8aa60e1bce9838.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            'getupstreammsgdistmonth',
            data={
                'begin_date': self._to_date_str(begin_date),
                'end_date': self._to_date_str(end_date)
            },
            result_processor=lambda x: x['list']
        )
        return res



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from wechatpyyyy.client.api.base import BaseWeChatAPI


class WeChatMisc(BaseWeChatAPI):

    def short_url(self, long_url):
        """
        将一条长链接转成短链接
        详情请参考
        http://mp.weixin.qq.com/wiki/10/165c9b15eddcfbd8699ac12b0bd89ae6.html

        :param long_url: 长链接地址
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.misc.short_url('http://www.qq.com')

        """
        return self._post(
            'shorturl',
            data={
                'action': 'long2short',
                'long_url': long_url
            }
        )

    def get_wechat_ips(self):
        """
        获取微信服务器 IP 地址列表

        :return: IP 地址列表

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            ips = client.misc.get_wechat_ips()

        """
        res = self._get(
            'getcallbackip',
            result_processor=lambda x: x['ip_list']
        )
        return res



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from wechatpyyyy.exceptions import WeChatException


class InvalidCorpIdException(WeChatException):

    def __init__(self, errcode=-40005, errmsg='Invalid corp_id'):
        super(InvalidCorpIdException, self).__init__(errcode, errmsg)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from wechatpyyyy.client.api.base import BaseWeChatAPI


class MerchantCommon(BaseWeChatAPI):

    API_BASE_URL = 'https://api.weixin.qq.com/'

    def upload_image(self, filename, image_data):
        res = self._post(
            'merchant/common/upload_img',
            params={
                'filename': filename
            },
            data=image_data,
            result_processor=lambda x: x['image_url']
        )
        return res



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import random
from datetime import datetime

from wechatpyyyy.pay.utils import get_external_ip
from wechatpyyyy.pay.base import BaseWeChatPayAPI


class WeChatMicroPay(BaseWeChatPayAPI):
    def create(self, body, total_fee, auth_code, client_ip=None, out_trade_no=None, detail=None, attach=None,
               fee_type='CNY', goods_tag=None, device_info=None, limit_pay=None):
        """
        刷卡支付接口
        :param device_info: 可选，终端设备号(商户自定义，如门店编号)
        :param body: 商品描述
        :param detail: 可选，商品详情
        :param attach: 可选，附加数据，在查询API和支付通知中原样返回，该字段主要用于商户携带订单的自定义数据
        :param client_ip: 可选，APP和网页支付提交用户端ip，Native支付填调用微信支付API的机器IP
        :param out_trade_no: 可选，商户订单号，默认自动生成
        :param total_fee: 总金额，单位分
        :param fee_type: 可选，符合ISO 4217标准的三位字母代码，默认人民币：CNY
        :param goods_tag: 可选，商品标记，代金券或立减优惠功能的参数
        :param limit_pay: 可选，指定支付方式，no_credit--指定不能使用信用卡支付
        :param auth_code: 授权码，扫码支付授权码，设备读取用户微信中的条码或者二维码信息
        :return: 返回的结果数据
        """
        now = datetime.now()
        if not out_trade_no:
            out_trade_no = '{0}{1}{2}'.format(
                self.mch_id,
                now.strftime('%Y%m%d%H%M%S'),
                random.randint(1000, 10000)
            )
        data = {
            'appid': self.appid,
            'device_info': device_info,
            'body': body,
            'detail': detail,
            'attach': attach,
            'out_trade_no': out_trade_no,
            'total_fee': total_fee,
            'fee_type': fee_type,
            'spbill_create_ip': client_ip or get_external_ip(),
            'goods_tag': goods_tag,
            'limit_pay': limit_pay,
            'auth_code': auth_code,
        }
        return self._post('pay/micropay', data=data)

    def query(self, transaction_id=None, out_trade_no=None):
        """
        查询订单

        :param transaction_id: 微信的订单号，优先使用
        :param out_trade_no: 商户系统内部的订单号，当没提供transaction_id时需要传这个。
        :return: 返回的结果数据
        """
        data = {
            'appid': self.appid,
            'transaction_id': transaction_id,
            'out_trade_no': out_trade_no,
        }
        return self._post('pay/orderquery', data=data)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from wechatpyyyy.pay.api.redpack import WeChatRedpack  # NOQA
from wechatpyyyy.pay.api.transfer import WeChatTransfer  # NOQA
from wechatpyyyy.pay.api.coupon import WeChatCoupon  # NOQA
from wechatpyyyy.pay.api.order import WeChatOrder  # NOQA
from wechatpyyyy.pay.api.refund import WeChatRefund  # NOQA
from wechatpyyyy.pay.api.tools import WeChatTools  # NOQA
from wechatpyyyy.pay.api.jsapi import WeChatJSAPI  # NOQA
from wechatpyyyy.pay.api.micropay import WeChatMicroPay  # NOQA
from wechatpyyyy.pay.api.withhold import WeChatWithhold  # NOQA



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from wechatpyyyy.client.base import BaseWeChatClient
from wechatpyyyy.enterprise.client import api


class WeChatClient(BaseWeChatClient):
    API_BASE_URL = 'https://qyapi.weixin.qq.com/cgi-bin/'

    agent = api.WeChatAgent()
    appchat = api.WeChatAppChat()
    batch = api.WeChatBatch()
    calendar = api.WeChatCalendar()
    chat = api.WeChatChat()
    department = api.WeChatDepartment()
    external_contact = api.WeChatExternalContact()
    external_contact_group_chat = api.WeChatExternalContactGroupChat()
    jsapi = api.WeChatJSAPI()
    material = api.WeChatMaterial()
    media = api.WeChatMedia()
    menu = api.WeChatMenu()
    message = api.WeChatMessage()
    misc = api.WeChatMisc()
    oauth = api.WeChatOAuth()
    schedule = api.WeChatSchedule()
    service = api.WeChatService()
    shakearound = api.WeChatShakeAround()
    tag = api.WeChatTag()
    user = api.WeChatUser()
    OA = api.WeChatOA()

    def __init__(self, corp_id, secret, access_token=None,
                 session=None, timeout=None, auto_retry=True):
        self.corp_id = corp_id
        self.secret = secret
        super(WeChatClient, self).__init__(
            corp_id, access_token, session, timeout, auto_retry
        )

    @property
    def access_token_key(self):
        return '{0}_{1}_access_token'.format(self.corp_id, self.secret[:10])

    def fetch_access_token(self):
        """ Fetch access token"""
        return self._fetch_access_token(
            url='https://qyapi.weixin.qq.com/cgi-bin/gettoken',
            params={
                'corpid': self.corp_id,
                'corpsecret': self.secret
            }
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from wechatpyyyy.client.api.base import BaseWeChatAPI


class WeChatPoi(BaseWeChatAPI):

    def add(self, poi_data):
        """
        创建门店

        详情请参考
        http://mp.weixin.qq.com/wiki/16/8f182af4d8dcea02c56506306bdb2f4c.html

        :param poi_data: 门店信息字典
        :return: 返回的 JSON 数据包
        """
        return self._post('poi/addpoi', data=poi_data)

    def get(self, poi_id):
        """
        查询门店信息

        详情请参考
        http://mp.weixin.qq.com/wiki/16/8f182af4d8dcea02c56506306bdb2f4c.html

        :param poi_id: 门店 ID
        :return: 返回的 JSON 数据包
        """
        return self._post('poi/getpoi', data={'poi_id': poi_id})

    def list(self, begin=0, limit=20):
        """
        查询门店列表

        详情请参考
        http://mp.weixin.qq.com/wiki/16/8f182af4d8dcea02c56506306bdb2f4c.html

        :param begin: 开始位置，0 即为从第一条开始查询
        :param limit: 返回数据条数，最大允许50，默认为20
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'poi/getpoilist',
            data={
                'begin': begin,
                'limit': limit,
            }
        )

    def update(self, poi_data):
        """
        修改门店

        详情请参考
        http://mp.weixin.qq.com/wiki/16/8f182af4d8dcea02c56506306bdb2f4c.html

        :param poi_data: 门店信息字典
        :return: 返回的 JSON 数据包
        """
        return self._post('poi/updatepoi', data=poi_data)

    def delete(self, poi_id):
        """
        删除门店

        详情请参考
        http://mp.weixin.qq.com/wiki/16/8f182af4d8dcea02c56506306bdb2f4c.html

        :param poi_id: 门店 ID
        :return: 返回的 JSON 数据包
        """
        return self._post('poi/delpoi', data={'poi_id': poi_id})

    def get_categories(self):
        """
        获取微信门店类目表

        详情请参考
        http://mp.weixin.qq.com/wiki/16/8f182af4d8dcea02c56506306bdb2f4c.html

        :return: 门店类目表
        """
        res = self._get(
            'api_getwxcategory',
            result_processor=lambda x: x['category_list']
        )
        return res



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from wechatpyyyy.client.api.base import BaseWeChatAPI


class WeChatService(BaseWeChatAPI):
    """
    应用授权（服务商、第三方应用开发相关）

    https://work.weixin.qq.com/api/doc#90001/90143/90597

    新的授权体系有部分接口未实现，欢迎提交 PR。
    """

    def get_provider_token(self, provider_secret):
        """
        获取服务商凭证

        https://work.weixin.qq.com/api/doc#90001/90143/91200

        :param provider_secret: 服务商的secret，在服务商管理后台可见
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'service/get_provider_token',
            data={
                'corpid': self._client.corp_id,
                'provider_secret': provider_secret,
            }
        )

    def get_suite_token(self, suite_id, suite_secret, suite_ticket):
        """
        获取第三方应用凭证

        https://work.weixin.qq.com/api/doc#90001/90143/9060

        :param suite_id: 以ww或wx开头应用id（对应于旧的以tj开头的套件id）
        :param suite_secret:  应用secret
        :param suite_ticket: 企业微信后台推送的ticket
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'service/get_suite_token',
            data={
                'suite_id': suite_id,
                'suite_secret': suite_secret,
                'suite_ticket': suite_ticket
            }
        )

    def get_login_info(self, auth_code, provider_access_token=None):
        """
        获取企业号登录用户信息

        详情请参考
        https://qydev.weixin.qq.com/wiki/index.php?title=获取企业号登录用户信息

        :param provider_access_token: 服务提供商的 accesstoken
        :param auth_code: OAuth 2.0 授权企业号管理员登录产生的 code
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'service/get_login_info',
            params={
                'provider_access_token': provider_access_token,
            },
            data={
                'auth_code': auth_code,
            }
        )

    def get_login_url(self, login_ticket, target, agentid=None, provider_access_token=None):
        """
        获取登录企业号官网的url

        详情请参考
        https://qydev.weixin.qq.com/wiki/index.php?title=获取登录企业号官网的url

        :param provider_access_token: 服务提供商的 accesstoken
        :param login_ticket: 通过get_login_info得到的login_ticket, 10小时有效
        :param target: 登录跳转到企业号后台的目标页面
        :param agentid: 可选，授权方应用id
        :return: 返回的 JSON 数据包
        """
        return self._post(
            'service/get_login_url',
            params={
                'provider_access_token': provider_access_token,
            },
            data={
                'login_ticket': login_ticket,
                'target': target,
                'agentid': agentid,
            }
        )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from wechatpyyyy.client.api.base import BaseWeChatAPI


class MerchantCategory(BaseWeChatAPI):

    API_BASE_URL = 'https://api.weixin.qq.com/'

    def get_sub_categories(self, cate_id):
        res = self._post(
            'merchant/category/getsub',
            data={'cate_id': cate_id},
            result_processor=lambda x: x['cate_list']
        )
        return res

    def get_sku_list(self, cate_id):
        res = self._post(
            'merchant/category/getsku',
            data={'cate_id': cate_id},
            result_processor=lambda x: x['sku_table']
        )
        return res

    def get_properties(self, cate_id):
        res = self._post(
            'merchant/category/getproperty',
            data={'cate_id': cate_id},
            result_processor=lambda x: x['properties']
        )
        return res



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import time
import logging
from wechatpyyyy.utils import random_string, to_text
from wechatpyyyy.pay.base import BaseWeChatPayAPI
from wechatpyyyy.pay.utils import calculate_signature

logger = logging.getLogger(__name__)


class WeChatJSAPI(BaseWeChatPayAPI):

    def get_jsapi_signature(self, prepay_id, timestamp=None, nonce_str=None):
        """
        获取 JSAPI 签名

        :param prepay_id: 统一下单接口返回的 prepay_id 参数值
        :param timestamp: 可选，时间戳，默认为当前时间戳
        :param nonce_str: 可选，随机字符串，默认自动生成
        :return: 签名
        """
        data = {
            'appId': self.sub_appid or self.appid,
            'timeStamp': timestamp or to_text(int(time.time())),
            'nonceStr': nonce_str or random_string(32),
            'signType': 'MD5',
            'package': 'prepay_id={0}'.format(prepay_id),
        }
        return calculate_signature(
            data,
            self._client.api_key if not self._client.sandbox else self._client.sandbox_api_key
        )

    def get_jsapi_params(self, prepay_id, timestamp=None, nonce_str=None, jssdk=False):
        """
        获取 JSAPI 参数

        :param prepay_id: 统一下单接口返回的 prepay_id 参数值
        :param timestamp: 可选，时间戳，默认为当前时间戳
        :param nonce_str: 可选，随机字符串，默认自动生成
        :param jssdk: 前端调用方式，默认使用 WeixinJSBridge
                      使用 jssdk 调起支付的话，timestamp 的 s 为小写
                      使用 WeixinJSBridge 调起支付的话，timeStamp 的 S 为大写
        :return: 参数
        """
        data = {
            'appId': self.sub_appid or self.appid,
            'timeStamp': timestamp or to_text(int(time.time())),
            'nonceStr': nonce_str or random_string(32),
            'signType': 'MD5',
            'package': 'prepay_id={0}'.format(prepay_id),
        }
        sign = calculate_signature(
            data,
            self._client.api_key if not self._client.sandbox else self._client.sandbox_api_key
        )
        logger.debug('JSAPI payment parameters: data = %s, sign = %s', data, sign)
        data['paySign'] = sign
        if jssdk:
            data['timestamp'] = data.pop('timeStamp')
        return data



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from wechatpyyyy.client.api.base import BaseWeChatAPI


class WeChatShakeAround(BaseWeChatAPI):

    def get_shake_info(self, ticket):
        """
        获取摇周边的设备及用户信息

        https://qydev.weixin.qq.com/wiki/index.php?title=获取设备及用户信息

        :param ticket: 摇周边业务的ticket，可在摇到的 URL 中得到，ticket 生效时间为30分钟
        :return: 设备及用户信息
        """
        res = self._post(
            'shakearound/getshakeinfo',
            data={
                'ticket': ticket
            }
        )
        return res['data']
